#!/usr/bin/env python

__RCSID__ = "$Id$"

# DIRAC imports
import DIRAC
from DIRAC import gLogger
from DIRAC.Core.Utilities.DIRACScript import DIRACScript as Script
from DIRAC.TransformationSystem.Client.TransformationClient import TransformationClient

Script.setUsageMessage("""
Treat the incomplete transfers by removing the replicas at source SEs but not at the destination SE

Usage:
   cta-prod-treat-incomplete-transfers <ascii file with list of (transID lfn)> <destSE>

Arguments:
  <ascii file with list of (transID lfn)> : input file with (transID lfn) to treat
  <destSE>                                : destination SE at which the Replica must be kept
""")

Script.parseCommandLine(ignoreErrors=True)

from DIRAC.DataManagementSystem.Client.DataManager import DataManager

def read_inputs(file_path):
  content = open(file_path, 'rt').readlines()
  resList = []
  for line in content:
    transID = line.strip().split(" ")[0]
    lfn = line.strip().split(" ")[1]
    if line != "\n":
      resList.append((transID, lfn))
  return resList

def removeSourceReplica(sourceSEs, lfn):
  dm = DataManager()
  for SE in sourceSEs:
    res = dm.removeReplica(SE, lfn)
    if not res['OK']:
      gLogger.error('Error removing source replicas for:', lfn)
      return res['Message']
  return DIRAC.S_OK()

@Script()
def main():
  args = Script.getPositionalArgs()

  if len(args) != 2:
    Script.showHelp()

  infile = args[0]
  destSE = args[1]
  resList = read_inputs(infile)
  tc = TransformationClient()

  sourceSEs = ['DESY-ZN-Disk', 'LPNHE-Disk', 'CNAF-Disk', 'CYF-STORM-Disk', 'LAPP-Disk', 'CEA-Disk', 'CC-IN2P3-Disk',
               'POLGRID-Disk', 'LANCASTER-Disk','CC-IN2P3-Tape','CNAF-Tape','DESY-ZN-Tape']

  if destSE in sourceSEs:
    sourceSEs.remove(destSE)

  # To do: improve by passing a list with all lfns to setFileStatusForTransformation
  for (transID, lfn) in resList:
    res = removeSourceReplica(sourceSEs, lfn)
    if not res['OK']:
      gLogger.error(res['Message'])
      DIRAC.exit(-1)
    res = tc.setFileStatusForTransformation(transID, newLFNsStatus='Processed', lfns=[lfn])
    if not res['OK']:
      gLogger.error(res['Message'])
      DIRAC.exit(-1)


if __name__ == '__main__':
  main()



#!/usr/bin/env python

"""
"""

__RCSID__ = "$Id$"

import os
from statistics import mean
from statistics import stdev

from DIRAC.Core.Utilities.DIRACScript import DIRACScript as Script

@Script()

def main():

  Script.parseCommandLine(ignoreErrors=True)

  #optionList = ['','with-scts-with-moon_']
  optionList = ['']
  particleList = ['gamma', 'proton']
  #particleList = ['gamma']
  #compilerList = ['gcc83_noOpt', 'gcc83_matchcpu']
  compilerList = ['gcc83_matchcpu']
  dirNameList = []
  for option in optionList:
    for particle in particleList:
      for compiler in compilerList:
        for cersiz in ['3', '5']:
          dirName = 'Paranal_%s_North_20_cersiz_%s_%s_%s01' % (particle, cersiz, compiler, option)
          dirNameList.append(dirName)

  for dirName in dirNameList:
    #print(dirName)
    if 'gamma' in dirName and 'with-scts-with-moon' not in dirName and 'cersiz_5' in dirName:
      string = 'S-/g bs 5:'
    if 'gamma' in dirName and 'with-scts-with-moon' not in dirName and 'cersiz_3' in dirName:
      string = 'S-/g bs 3:'
    if 'proton' in dirName and 'with-scts-with-moon' not in dirName and 'cersiz_5' in dirName:
      string = 'S-/p bs 5:'
    if 'proton' in dirName and 'with-scts-with-moon' not in dirName and 'cersiz_3' in dirName:
      string = 'S-/p bs 3:'
    if 'gamma' in dirName and 'with-scts-with-moon' in dirName and 'cersiz_5' in dirName:
      string = 'S+/g bs 5:'
    if 'gamma' in dirName and 'with-scts-with-moon' in dirName and 'cersiz_3' in dirName:
      string = 'S+/g bs 3:'
    if 'proton' in dirName and 'with-scts-with-moon' in dirName and 'cersiz_5' in dirName:
      string = 'S+/p bs 5:'
    if 'proton' in dirName and 'with-scts-with-moon' in dirName and 'cersiz_3' in dirName:
      string = 'S+/p bs 3:'
    wallTimeList = []
    for subdir in os.listdir(dirName):
      for subsubdir in os.listdir(os.path.join(dirName,subdir)):
        fileName = os.path.join(dirName,subdir,subsubdir,'CorsikaSimtel_Log.txt')
        if os.path.exists(fileName):
          file = open(fileName, 'r')
          for line in file:
            if 'Multipipe_corsika finished after' in line:
              wallTime = line.split('Multipipe_corsika finished after ')[1].split(' seconds run-time.')[0]
              #print (wallTime)
              wallTimeList.append(float(wallTime))

    print( "%s  %.2f +- %.2f s" % (string, mean(wallTimeList), stdev(wallTimeList)))

if __name__ == "__main__":
  main()



"""
    Launcher script to launch a production with 2 steps
    simulation Step: Prod5bMCPipeNSBJob
    calibimgreco Step: EvnDispProd5SingJob
"""

__RCSID__ = "$Id$"

import json
from copy import copy

from DIRAC.Core.Base import Script
Script.parseCommandLine()
import DIRAC

from DIRAC.ProductionSystem.Client.ProductionClient import ProductionClient
from DIRAC.ProductionSystem.Client.ProductionStep import ProductionStep
from CTADIRAC.Interfaces.API.Prod5bMCPipeNSBJob import Prod5bMCPipeNSBJob
from CTADIRAC.Interfaces.API.EvnDispProd5SingJob import EvnDispProd5SingJob
from CTADIRAC.Core.Utilities.tool_box import get_dataset_MQ
from DIRAC.Core.Workflow.Parameter import Parameter


def build_simulation_step(DL0_data_set, name_tag=''):
  ''' Setup Corsika + sim_telarray step

  Note that there is no InputQuery,
  since jobs created by this steps don't require any InputData

  @return ProductionStep object
  '''
  # Note that there is no InputQuery,
  # since jobs created by this steps don't require any InputData
  prod_step_1 = ProductionStep()
  prod_step_1.Name = 'Simulation_%s' % DL0_data_set.replace('AdvancedBaseline_NSB1x_', '')
  prod_step_1.Name += '%s' % name_tag
  DIRAC.gLogger.notice('\tBuilding MC Production step: %s' % prod_step_1.Name)
  prod_step_1.Type = 'MCSimulation'
  prod_step_1.Outputquery = get_dataset_MQ(DL0_data_set)
  prod_step_1.Outputquery['nsb'] = {'in': [1, 5]}

  # get meta data to be passed to simulation job
  site = prod_step_1.Outputquery['site']
  particle = prod_step_1.Outputquery['particle']
  if prod_step_1.Outputquery['phiP']['='] == 180:
    pointing_dir = 'North'
  elif prod_step_1.Outputquery['phiP']['='] == 0:
    pointing_dir = 'South'
  zenith_angle = prod_step_1.Outputquery['thetaP']['=']

  # Here define the job description (i.e. Name, Executable, etc.)
  # to be associated to the first ProductionStep, as done when using the TS
  job1 = Prod5bMCPipeNSBJob()
  job1.version = '2020-06-29b'
  job1.compiler = 'gcc83_matchcpu'
  # Initialize JOB_ID
  job1.workflow.addParameter(Parameter("JOB_ID", "000000", "string", "", "",
                                       True, False, "Temporary fix"))
  # configuration
  job1.setName('Prod5b_MC_Pipeline_NSB')
  job1.set_site(site)
  job1.set_particle(particle)
  job1.set_pointing_dir(pointing_dir)
  job1.zenith_angle = zenith_angle
  job1.n_shower = 50000
  if particle == 'gamma':
    job1.n_shower = 20000

  job1.setOutputSandbox(['*Log.txt'])
  job1.start_run_number = '0'
  job1.run_number = '@{JOB_ID}'  # dynamic
  job1.output_file_size = 100000  # 200 MB
  job1.setupWorkflow(debug=False)
  # Add the job description to the first ProductionStep
  prod_step_1.Body = job1.workflow.toXML()
  # return ProductionStep object
  return prod_step_1


def build_evndisp_step(DL0_data_set, nsb=1, name_tag=''):
  ''' Define a new EventDisplay analysis production step

  @return ProductionStep object
  '''
  if nsb == 1:
    DL0_data_set_NSB = DL0_data_set
  elif nsb == 5:
    DL0_data_set_NSB = DL0_data_set.replace('NSB1x', 'NSB5x')

  prod_step_2 = ProductionStep()
  prod_step_2.Name = 'Analysis_' + DL0_data_set_NSB.replace('AdvancedBaseline_', '').replace('DL0', 'DL1')
  prod_step_2.Name += '%s' % name_tag
  DIRAC.gLogger.notice('\tBuilding Analysis Production step: %s' % prod_step_2.Name)
  prod_step_2.Type = 'DataReprocessing'  # This corresponds to the Transformation Type
  prod_step_2.Inputquery = get_dataset_MQ(DL0_data_set_NSB)
  prod_step_2.Outputquery = get_dataset_MQ(DL0_data_set_NSB.replace('DL0', 'DL1'))

  # Here define the job description to be associated to the second ProductionStep
  job2 = EvnDispProd5SingJob(cpuTime=259200.)
  job2.version = 'eventdisplay-cta-dl1-prod5.v04'
  job2.setName('Prod5b_EvnDisp_Singularity')
  # output
  job2.setOutputSandbox(['*Log.txt'])
  # refine output meta data if needed
  output_meta_data = copy(prod_step_2.Outputquery)
  job2.set_meta_data(output_meta_data)
  job2.set_file_meta_data(nsb=output_meta_data['nsb']['='])

  job2.ts_task_id = '@{JOB_ID}'  # dynamic
  job2.group_size = 5   # for the input files verification
  job2.setupWorkflow(debug=False)
  job2.setType('EvnDisp3')  # mandatory *here*
  prod_step_2.Body = job2.workflow.toXML()
  prod_step_2.GroupSize = 5  # have to match the above group size?
  # return ProductionStep object
  return prod_step_2


#########################################################
if __name__ == '__main__':
  # get arguments
  tag = ''
  args = Script.getPositionalArgs()
  if len(args) < 1:
    DIRAC.gLogger.error('At least 1 argument required: DL0_data_set_name')
    DIRAC.exit(-1)
  DL0_data_set = args[0]
  if len(args) == 2:
    tag = args[1]
  prod_name = DL0_data_set.replace('AdvancedBaseline_NSB1x_', '') + '_DL1'
  if tag != '':
    prod_name += '-%s' % tag

  ##################################
  # Create the production
  DIRAC.gLogger.notice('Building new production: %s' % prod_name)
  prod_sys_client = ProductionClient()

  ##################################
  # Define the first ProductionStep (Corsika+sim_telarray)
  prod_step_1 = build_simulation_step(DL0_data_set, name_tag=tag)
  # Add the step to the production
  prod_sys_client.addProductionStep(prod_step_1)

  ##################################
  # Define EventDisplay analysis steps and add them to the production
  # dark nsb = 1
  prod_step_2 = build_evndisp_step(DL0_data_set, nsb=1, name_tag=tag)
  prod_step_2.ParentStep = prod_step_1
  prod_sys_client.addProductionStep(prod_step_2)
  # moon nsb = 5
  prod_step_3 = build_evndisp_step(DL0_data_set, nsb=5, name_tag=tag)
  prod_step_3.ParentStep = prod_step_1
  prod_sys_client.addProductionStep(prod_step_3)

  ##################################
  # Get the production description
  prod_description = prod_sys_client.prodDescription

  # Create the production
  DIRAC.gLogger.notice('Creating production.')
  res = prod_sys_client.addProduction(prod_name, json.dumps(prod_description))
  if not res['OK']:
    DIRAC.gLogger.error(res['Message'])
    DIRAC.exit(-1)

  # Start the production, i.e. instantiate the transformation steps
  res = prod_sys_client.startProduction(prod_name)

  if not res['OK']:
    DIRAC.gLogger.error(res['Message'])
    DIRAC.exit(-1)

  DIRAC.gLogger.notice('Production %s successfully created' % prod_name)



#!/usr/bin/env python
"""
Get statistics for one or more datasets

Warning: --SEUsage option gives the storage report per SE
         and it assumes that datasets are defined with directory level metadata only
Usage:
   cta-prod-show-dataset <datasetName (may contain wild cards) or ascii file with a list of datasets> <options>
"""

__RCSID__ = "$Id$"

import os

import DIRAC
from DIRAC import gLogger
from DIRAC.Core.Utilities.DIRACScript import DIRACScript as Script
from DIRAC.Core.Utilities.PrettyPrint import int_with_commas, printTable
from DIRAC.Resources.Catalog.FileCatalogClient import FileCatalogClient
from CTADIRAC.Core.Utilities.tool_box import read_inputs_from_file

def print_dataset_list(dataset_list):
  dataset_list.sort()
  print('\nAvailable datasets: %d\n' % len(dataset_list))
  for dataset_name in dataset_list:
    gLogger.notice(dataset_name)
  gLogger.notice('\n')

def get_list_of_datasets(tag=''):
  fc = FileCatalogClient()
  dataset_tag = '*%s*' % tag
  res = fc.getDatasets(dataset_tag)
  if not res['OK']:
    gLogger.error(res['Message'])
    DIRAC.exit(-1)
  dataset_dict = res['Value']
  dataset_list = list()
  for dataset_name in dataset_dict['Successful'][dataset_tag]:
    dataset_list.append(dataset_name)
  return dataset_list

def get_dataset_info(dataset_name):
  fc = FileCatalogClient()
  res = fc.getDatasets(dataset_name)
  if not res['OK']:
    gLogger.error("Failed to get datasets")
    DIRAC.exit(-1)
  dataset_dict = res['Value']
  res = dataset_dict['Successful'][dataset_name][dataset_name]
  number_of_files = res['NumberOfFiles']
  meta_query = res['MetaQuery']
  total_size = res['TotalSize']
  return (dataset_name, number_of_files, total_size, meta_query)
  
def print_dataset_storage_usage(dataset_name):
  fc = FileCatalogClient()
  fields = ["StorageElement", "Size", "Replicas"]
  totalSize = 0.
  totalFiles = 0
  name, n_files, size, mq = get_dataset_info(dataset_name)
  outputType = mq['outputType']
  result = fc.findDirectoriesByMetadata(mq)
  if not result['OK']:
    gLogger.error(result['Message'])
    DIRAC.exit(-1)
  gLogger.notice('Directory size')
  for k, path in result['Value'].items():
    if os.path.basename(path) == outputType:
      gLogger.notice(path)
      result = fc.getDirectorySize(path,longOut=True)
      values = []
      res_path_dict = result["Value"]["Successful"][path]
      if "PhysicalSize" in res_path_dict:
        if "TotalSize" in res_path_dict["PhysicalSize"]:
          totalSize = res_path_dict["PhysicalSize"]["TotalSize"]
        if "TotalFiles" in res_path_dict["PhysicalSize"]:
          totalFiles = res_path_dict["PhysicalSize"]["TotalFiles"]
        for se, sdata in result["Value"]["Successful"][path]["PhysicalSize"].items():
          if not se.startswith("Total"):
            size = sdata["Size"]
            nfiles = sdata["Files"]
            values.append((se, int_with_commas(size), str(nfiles)))
        values.append(("Total", int_with_commas(totalSize), str(totalFiles)))
        printTable(fields, values)
 
def print_dataset_storage_usage_summary(path_list):
  fc = FileCatalogClient()
  fields = ["StorageElement", "Size", "Replicas"]
  values = []
  se_list = []
  stat_dict = {}
  total_stat_dict = {'Total':{'size':0,'nFiles':0}}

  # Initialize the list of SE for all paths
  for path in path_list:
    result = fc.getDirectorySize(path,longOut=True)
    if "PhysicalSize" in result["Value"]["Successful"][path]:
      for se, sdata in result["Value"]["Successful"][path]["PhysicalSize"].items():
        if not se.startswith("Total"):
          se_list.append(se)

  # Initialize the total_stat_dict with all SE keys        
  se_list = sorted(set(se_list))    
  for se in se_list:
    total_stat_dict.update({se:{'size':0,'nFiles':0}})
    stat_dict.update({se:{'size':0.,'nFiles':0}})

  # For each path get the directory usage and calculate the total usage for each SE
  # and the overall total usage 
  for path in path_list:
    for se in se_list:
      stat_dict[se]['size'] = 0
      stat_dict[se]['nFiles'] = 0
    result = fc.getDirectorySize(path,longOut=True)
    res_path_dict = result["Value"]["Successful"][path]
    if "PhysicalSize" in res_path_dict:
      for se, sdata in result["Value"]["Successful"][path]["PhysicalSize"].items():
        if not se.startswith("Total"):
          stat_dict[se]['size'] = sdata["Size"]
          stat_dict[se]['nFiles'] = sdata["Files"]

    # Calculate the total usage for each SE  
    for se in stat_dict:
      total_stat_dict[se]['size'] +=  stat_dict[se]['size']
      total_stat_dict[se]['nFiles'] +=  stat_dict[se]['nFiles']

  # Calculate the overall total usage
  for se in stat_dict:
    total_stat_dict['Total']['size'] += total_stat_dict[se]['size']
    total_stat_dict['Total']['nFiles'] += total_stat_dict[se]['nFiles']

  # Update the values of the table for each SE
  for se in se_list:
    values.append((se, int_with_commas(total_stat_dict[se]['size']), str(total_stat_dict[se]['nFiles']))) 

  # Update the values of the table with the Total
  values.append(("Total", int_with_commas(total_stat_dict['Total']['size']), str(total_stat_dict['Total']['nFiles']))) 

  printTable(fields, values)
  

#########################################################
@Script()
def main():
  Script.registerSwitch("l", "long", "print dataset details")
  Script.registerSwitch("", "SEUsage", "print storage usage summary per SE")
  switches, argss = Script.parseCommandLine(ignoreErrors=True)

  if len(argss) == 0:
    dataset_list = get_list_of_datasets()
    print_dataset_list(dataset_list)
    DIRAC.exit()
  elif len(argss) == 1:
    if os.path.isfile(argss[0]):
      gLogger.notice('Reading datasets from input file:', argss[0])
      dataset_list = read_inputs_from_file(argss[0])
    else:
      dataset_name = argss[0]
      dataset_list = list()
      if dataset_name.find('*') > 0:
        dataset_list = get_list_of_datasets(dataset_name)
      else:
        dataset_list.append(dataset_name)
    print_dataset_list(dataset_list)

  long = False
  SEUsage = False
  for switch in switches:
    if 'l' == switch[0] or 'long' == switch[0]:
      long = True
    if 'SEUsage' == switch[0]:
      SEUsage = True
    
  if SEUsage:
    gLogger.notice('Getting statistics informations for all datasets\n')
    
  # Get dataset info only if SE usage report or dataset details are requested
  if SEUsage or long:
    path_list = []
    fc = FileCatalogClient()

    ## Check FileMetadata
    res = fc.getMetadataFields()
    if not res['OK']:
      gLogger.error(res['Message'])
      DIRAC.exit(-1)
    fileMetaFields = res["Value"]["FileMetaFields"]

    i=1
    for dataset_name in dataset_list:
      name, n_files, size, mq = get_dataset_info(dataset_name)
      if long:
        gLogger.notice('%d) Dataset name: %s\n' % (i, name))
        gLogger.notice('MetaQuery')
        gLogger.notice(str(mq),'\n')
        i+=1
      # Get the path list only if the SE usage report is requested  
      if SEUsage:
        for metadata in mq:
          if metadata in fileMetaFields:
            gLogger.error('Warning: File level metadata found:', metadata)
            gLogger.error('Storage report per SE could be wrong for dataset:', dataset_name )
        if long:
          print_dataset_storage_usage(dataset_name)
        if 'outputType' in mq:
          outputType = mq['outputType']
        else:
          gLogger.error('Error getting SEUsage, no outputType metadata defined')
          gLogger.error('for dataset:', name)
          DIRAC.exit(-1)
        result = fc.findDirectoriesByMetadata(mq)
        for k, path in result['Value'].items():
          if os.path.basename(path) == outputType:
            path_list.append(path)

  if SEUsage:
    gLogger.notice('\nStorage usage per SE for all datasets\n')
    print_dataset_storage_usage_summary(path_list)

  # To do: optimize to avoid calling twice get_dataset_info when using SEUsage or long options
  gLogger.notice('Storage usage for all datasets\n')
  gLogger.notice('|_. Name |_. N files |_. Size(TB) |')
  total_size = 0.
  total_n_files = 0
  for dataset_name in dataset_list:
    name, n_files, size, mq = get_dataset_info(dataset_name)
    # # convert total size in TB
    size_TB = size / 1e12
    gLogger.notice('|%s|%d|%.2f|' % (name, n_files, size_TB))
    total_size += size_TB
    total_n_files += n_files
  gLogger.notice('| Total | %d | %.2f |\n' % (total_n_files, total_size))
    
  DIRAC.exit()

if __name__ == '__main__':
  main()



""" Module to setup Prod3 MC software
"""

__RCSID__ = "$Id$"

# generic imports
import os
import tarfile

# DIRAC imports
import DIRAC
from DIRAC.DataManagementSystem.Client.DataManager import DataManager
from DIRAC.ConfigurationSystem.Client.Helpers.Operations import Operations

# @TODO
# handle different OS platform for the transition to CentOS7
# >>> import platform
# >>> platform.linux_distribution()
# ('CentOS Linux', '7.3.1611', 'Core')
# >>> platform.linux_distribution()
# ('Scientific Linux', '6.9', 'Carbon')

class Prod3SoftwareManager(object) :
  """ Manage software setup for prod3
  """

  def __init__( self, soft_category ):
    """ Constructor
    """
    self.SW_SHARED_DIR = 'VO_VO_CTA_IN2P3_FR_SW_DIR'
    self.CVMFS_DIR = '/cvmfs/cta.in2p3.fr/software'
    self.LFN_ROOT = '/vo.cta.in2p3.fr/software'
    self.SOFT_CATEGORY_DICT = soft_category
    self.dm = DataManager()

  def installDIRACScripts( self, package_dir ):
    """ copy prod3 DIRAC scripts in the current directory
    """
    cmd = 'cp ' + os.path.join( package_dir, 'dirac_*' ) + ' .'
    if not os.system( cmd ):
      return DIRAC.S_OK()
    else:
      return DIRAC.S_ERROR( 'Failed to install DIRAC scripts' )

  def dumpSetupScriptPath( self, package_dir, textfilename = 'setup_script_path.txt' ):
    """ dump the path to setupPackage.sh in a one line ascii file
          to be read and source by the following script
    """
    script_path = os.path.join( package_dir, 'setupPackage.sh' )
    open( textfilename, 'w' ).writelines( script_path + '\n' )
    return DIRAC.S_OK()

  def installSoftwarePackage( self, package, version, arch = "sl6-gcc44", installDir = '.' ):
    """ install software package in the current directory
    """
    DIRAC.gLogger.notice( 'Installing package %s version %s' % ( package, version ) )
    tarFile = package + '.tar.gz'
    tarLFN = os.path.join( self.LFN_ROOT, package, version, tarFile )

    ########## download the tar file #######################
    DIRAC.gLogger.notice( 'Trying to download package:', tarLFN )
    res = self.dm.getFile( tarLFN )
    if not res['OK']:
      return res

    if tarLFN in res['Value']['Successful']:
      DIRAC.gLogger.notice( ' Package downloaded successfully:', tarLFN )
    else:
      error = 'Failed to download package:', tarLFN
      return DIRAC.S_ERROR( error )

    ########## extract the tar file #######################
    tarMode = "r|*"
    tar = tarfile.open( tarFile, tarMode )
    for tarInfo in tar:
      tar.extract( tarInfo, installDir )
    tar.close()
    os.unlink( tarFile )

    DIRAC.gLogger.notice( 'Package %s version %s installed successfully at:\n%s' % ( package, version, installDir ) )

    return DIRAC.S_OK( installDir )

  def _getSoftwareAreas( self ):
    """ get the list of available software areas (shared area, cvmfs)
    """
    areaList = []

    opsHelper = Operations()
    UseCvmfs = opsHelper.getValue( 'SoftwarePolicy/UseCvmfs', bool )
    DIRAC.gLogger.notice( 'SoftwarePolicy for UseCvmfs is:', UseCvmfs )
    if UseCvmfs:
      areaList.append( self.CVMFS_DIR )

    if self.SW_SHARED_DIR in os.environ:
      shared_area = os.path.join( os.environ[self.SW_SHARED_DIR], 'software' )
      areaList.append( shared_area )
    else:
      DIRAC.gLogger.warn( 'Shared area not found' )

    return areaList

  def _getSharedArea( self ):
    """ get Shared Area
    """
    if self.SW_SHARED_DIR in os.environ:
      shared_area = os.path.join( os.environ[self.SW_SHARED_DIR], 'software' )
    else:
      return DIRAC.S_ERROR( 'Shared area not found' )

    return DIRAC.S_OK( shared_area )

  def _getPackageDir( self, area, arch, package, version ):
    """ get Package directory
    """
    package_dir = os.path.join( area, arch, self.SOFT_CATEGORY_DICT[package], package, version )
    return package_dir

  def removeSoftwarePackage( self, packagedir ):
    """ remove Software Package
    """
    cmd = 'rm -Rf ' + packagedir
    if not os.system( cmd ):
      return DIRAC.S_OK()
    else:
      error = 'Failed to remove %s' % packagedir
      return DIRAC.S_ERROR( error )


  def checkSoftwarePackage( self, package, version, arch = "sl6-gcc44", area = None ):
    """ check if the software package is installed in any software area
      Keyword arguments:
      package -- package name as the directory name
      version -- software version as the directory name
      arch -- architecture as the directory name
    """

    # if area is specified, just get the shared area
    if area:
      areaList = self._getSharedArea()
    else:
      areaList = self._getSoftwareAreas()

    if len( areaList ) == 0:
      DIRAC.gLogger.warn( 'No software area is available' )

    # ## look for the package directory in the software areas
    for area in areaList:
      package_dir = os.path.join( area, arch, self.SOFT_CATEGORY_DICT[package], package, version )
      if os.path.isdir( package_dir ):
        DIRAC.gLogger.notice( 'Found package %s version %s at:\n%s' % ( package, version, package_dir ) )
        return DIRAC.S_OK( package_dir )

    return DIRAC.S_ERROR( 'Could not find package %s version %s in any location' % ( package, version ) )



#!/usr/bin/env python
"""
Clone a given branch of a git repository
branch is optional
Usage:
   cta-prod-git-clone <repo_url> <branch>
"""

__RCSID__ = "$Id$"

# generic imports
from git import Repo

# DIRAC imports
import DIRAC
from DIRAC.Core.Utilities.DIRACScript import DIRACScript as Script

@Script()
def main():
  Script.parseCommandLine(ignoreErrors=True)
  args = Script.getPositionalArgs()
  branch = None
  if len(args) == 1:
    repo_url = args[0]
  elif len(args) == 2:
    repo_url = args[0]
    branch = args[1]
  else:
    Script.showHelp()

  git_clone(repo_url, branch)
  DIRAC.exit()

def git_clone(repo_url, branch=None):
  ''' Clone a given branch of a git repo '''
  if branch is None:
    Repo.clone_from(repo_url, repo_url.split("/")[-1])
  else:
    Repo.clone_from(repo_url, repo_url.split("/")[-1], branch=branch)


if __name__ == '__main__':
  main()




#!/usr/bin/env python

"""
  Get tasks attached to a transformation for a given TaskStatus
"""

__RCSID__ = "$Id$"

import DIRAC
from DIRAC import gLogger
from DIRAC.Core.Utilities.DIRACScript import DIRACScript as Script
from DIRAC.TransformationSystem.Client.TransformationClient import TransformationClient

@Script()
def main():
  Script.registerSwitch('', 'TransID=', 'transformation ID')
  Script.registerSwitch('', 'Status=', 'task status')
  Script.parseCommandLine()

  transID = None
  status = None

  for switch in Script.getUnprocessedSwitches():
    if switch[0] == 'TransID':
      try:
        transID = int(switch[1])
      except Exception:
        gLogger.fatal('Invalid transID', switch[1])
    elif switch[0] == 'Status':
      status = switch[1].capitalize()

  if not transID:
    Script.showHelp(exitCode=2)

  tc = TransformationClient()

  condDict = {'TransformationID': transID}
  if status:
    condDict.update({'ExternalStatus': status})

  res = tc.getTransformationTasks(condDict)

  if not res['OK']:
    DIRAC.gLogger.error(res['Message'])
    DIRAC.exit(2)

  if len(res['Value']) == 0:
    DIRAC.gLogger.notice("No tasks selected for transformation %s with status %s" % (transID, status))

  for task in res['Value']:
    DIRAC.gLogger.notice(task["TaskID"])

if __name__ == "__main__":
  main()


#!/usr/bin/env python

"""
  Delete a transformation or a list of transformations

  Usage:
    cta-transformation-delete <list of transID comma separated OR ascii file containing a list of transID (comma-separated on each line)>
"""

__RCSID__ = "$Id$"

import os

import DIRAC
from DIRAC.Core.Utilities.DIRACScript import DIRACScript as Script
from DIRAC.TransformationSystem.Client.TransformationClient import TransformationClient

@Script()
def main():
    Script.parseCommandLine()
    args = Script.getPositionalArgs()
    if (len(args) != 1):
      Script.showHelp()

    # get arguments
    transIDs = []
    for arg in args[0].split(','):
      if os.path.exists(arg):
        lines = open(arg, 'rt').readlines()
        for line in lines:
          for transID in line.split(','):
            transIDs += [int(transID.strip())]
      else:
        transIDs.append(int(arg))

    tc = TransformationClient()

    for transID in transIDs:
      res = tc.deleteTransformation(transID)
      if not res['OK']:
        DIRAC.gLogger.error('Failed to delete transformation %s: %s' % (transID, res['Message']))
        continue
      else:
        DIRAC.gLogger.notice("Successfully deleted transformation %s" % transID)

    DIRAC.exit(0)

if __name__ == "__main__":
  main()


"""
    Launcher script for a simple job example
"""

__RCSID__ = "$Id$"

from DIRAC.Core.Base import Script
Script.parseCommandLine()
from DIRAC.Interfaces.API.Job import Job
from DIRAC import gLogger
from DIRAC.Interfaces.API.Dirac import Dirac

dirac = Dirac()
j = Job()
j.setCPUTime(500)
j.setExecutable('ls')
j.setName('testjob')
res = dirac.submitJob(j)
gLogger.notice('Submission Result:', res['Value'])



# imports
import six
import json
#from types import StringTypes
# Import sqlachemy modules to create objects mapped with tables
from sqlalchemy import Table, Column, ForeignKey
from sqlalchemy import Integer, String
from sqlalchemy import exists
from sqlalchemy.orm import sessionmaker, class_mapper, relationship
from sqlalchemy.orm.exc import NoResultFound
from sqlalchemy.engine.reflection import Inspector
from sqlalchemy import create_engine, func, MetaData, \
    Integer, String, DateTime, Enum, BLOB, exc, BigInteger, distinct

# Declare a declarative_base to map objets and tables
from sqlalchemy.ext.declarative import declarative_base

# from DIRAC
from DIRAC import S_OK, S_ERROR, gLogger, gConfig
from DIRAC.ConfigurationSystem.Client.Utilities import getDBParameters


provBase = declarative_base()

################################################################################
# wasInformedBy association table (n-n relation)
wasInformedBy_association_table = Table('wasInformedBy', provBase.metadata,
                                        Column('internal_key', BigInteger, primary_key=True, autoincrement=True),
                                        Column('informant', BigInteger, ForeignKey("activities.internal_key")),
                                        Column('informed', BigInteger, ForeignKey("activities.internal_key")))

################################################################################
# Define the Activity class mapped to the activities table


class Activity(provBase):
  __tablename__ = 'activities'
  ordered_attribute_list = ['id', 'name', 'startTime', 'endTime', 'comment',
                            'activityDescription_id']
  other_display_attributes = ['name', 'comment']

  # Internal key
  internal_key = Column(BigInteger, primary_key=True, autoincrement=True)

  # Model attributes
  id = Column(String)
  name = Column(String)
  startTime = Column(String)
  endTime = Column(String)
  comment = Column(String)

  # n-1 relation with ActivityDescription
  activityDescription_key = Column(BigInteger, ForeignKey("activityDescriptions.internal_key"))
  activityDescription = relationship("ActivityDescription")
  # n-n relation
  wasInformedBy = relationship('Activity',
                               secondary=wasInformedBy_association_table,
                               primaryjoin=internal_key == wasInformedBy_association_table.c.informed,
                               secondaryjoin=internal_key == wasInformedBy_association_table.c.informant)

  # Print method
  def __repr__(self):
    response = ""
    for attribute in self.ordered_attribute_list:
      response += "Activity.%s=%s\n" % (attribute, self.__dict__[attribute])
    return response

  # Other methods
  def get_display_attributes(self):
    response = {}
    for attribute in self.other_display_attributes:
      response['voprov:' + attribute] = self.__dict__[attribute]
    return response

  def get_description_key(self):
    return self.activityDescription_Key


################################################################################
# wasDerivedFrom association table (n-n relation)
wasDerivedFrom_association_table = Table('wasDerivedFrom', provBase.metadata,
                                         Column('internal_key', BigInteger, primary_key=True),
                                         Column('generatedEntity', BigInteger, ForeignKey("entities.internal_key")),
                                         Column('usedEntity', BigInteger, ForeignKey("entities.internal_key")))

################################################################################
# Define the Entity class mapped to the entities table


class Entity(provBase):
  __tablename__ = 'entities'
  ordered_attribute_list = ['id', 'classType', 'name', 'location', 'generatedAtTime',
                            'invalidatedAtTime', 'comment', 'entityDescription_id']
  other_display_attributes = ['id', 'name', 'location', 'generatedAtTime',
                              'invalidatedAtTime', 'comment']

  # Internal key
  internal_key = Column(BigInteger, primary_key=True, autoincrement=True)

  # Model attributes
  id = Column(String)
  name = Column(String)
  location = Column(String)
  generatedAtTime = Column(DateTime)
  invalidatedAtTime = Column(DateTime)
  comment = Column(String)

  # Heritage
  classType = Column(String)
  __mapper_args__ = {
      'polymorphic_identity': 'entity',
      'polymorphic_on': classType
  }

  # n-1 relation with EntityDescription
  entityDescription_key = Column(BigInteger, ForeignKey("entityDescriptions.internal_key"))
  entityDescription = relationship("EntityDescription")
  # n-n relation
  wasDerivedFrom = relationship('Entity',
                                secondary=wasDerivedFrom_association_table,
                                primaryjoin=internal_key == wasDerivedFrom_association_table.c.usedEntity,
                                secondaryjoin=internal_key == wasDerivedFrom_association_table.c.generatedEntity)

  # Print method
  def __repr__(self):
    response = ""
    for attribute in self.ordered_attribute_list:
      response += "Entity.%s=%s\n" % (attribute, self.__dict__[attribute])
    return response

  # Other methods
  def get_display_attributes(self):
    response = {}
    for attribute in self.other_display_attributes:
      response['voprov:' + attribute] = self.__dict__[attribute]
    return response


################################################################################
# Define the collection class withy the hadMember: TO BE DONE!

################################################################################
# Define the ValueEntity class mapped to the valueEntities table
class ValueEntity(Entity):
  __tablename__ = 'valueEntities'
  # ordered_attribute_list = Entity.ordered_attribute_list+['value']
  ordered_attribute_list = Entity.ordered_attribute_list
  other_display_attributes = ['value']

  # Internal key
  internal_key = Column(BigInteger, ForeignKey('entities.internal_key'), primary_key=True)

  # Model attributes
  value = Column(String)

  # Heritage
  __mapper_args__ = {'polymorphic_identity': 'value'}

  # Print method
  def __repr__(self):
    response = ""
    for attribute in self.ordered_attribute_list:
      response += "ValueEntity.%s=%s\n" % (attribute, self.__dict__[attribute])
    return response

  # Other methods
  def get_display_attributes(self):
    response = {}
    for attribute in self.other_display_attributes:
      response['voprov:' + attribute] = self.__dict__[attribute]
    return response

####################################################################################################
# Define the DatasetEntity class mapped to the datasetEntities table


class DatasetEntity(Entity):
  __tablename__ = 'datasetEntities'
  ordered_attribute_list = Entity.ordered_attribute_list
  other_display_attributes = ['ctadirac_guid']

  # Internal key
  internal_key = Column(BigInteger, ForeignKey('entities.internal_key'), primary_key=True)

  # Model attributes
  # CTADIRAC attributes
  ctadirac_guid = Column(String)

  # Heritage
  __mapper_args__ = {'polymorphic_identity': 'dataset'}

  # Print method
  def __repr__(self):
    response = ""
    for attribute in self.ordered_attribute_list:
      response += "DatasetEntity.%s=%s\n" % (attribute, self.__dict__[attribute])
    return response

  # Other methods
  def get_display_attributes(self):
    response = {}
    for attribute in self.other_display_attributes:
      response['voprov:' + attribute] = self.__dict__[attribute]
    return response

################################################################################
# Define the Used class mapped to the used table


class Used(provBase):
  __tablename__ = 'used'
  ordered_attribute_list = ['role', 'time', 'activity_key', 'entity_key',
                            'usageDescription_key']
  other_display_attributes = ['role', 'time']

  # Internal key
  internal_key = Column(BigInteger, primary_key=True, autoincrement=True)

  # Model attributes
  role = Column(String, nullable=True)
  time = Column(String)

  # n-1 relation with Activity
  activity_key = Column(BigInteger, ForeignKey('activities.internal_key'))
  activity = relationship("Activity", backref='used')
  # n-1 relation with Entity
  entity_key = Column(BigInteger, ForeignKey('entities.internal_key'))
  entity = relationship("Entity", backref='used')
  # n-1 relation with UsageDescription
  usageDescription_key = Column(BigInteger, ForeignKey('usageDescriptions.internal_key'))
  usageDescription = relationship("UsageDescription")

  # Print method
  def __repr__(self):
    response = ""
    for attribute in self.ordered_attribute_list:
      response += "Used.%s=%s\n" % (attribute, self.__dict__[attribute])
    return response

  # Other methods
  def get_display_attributes(self):
    response = {}
    for attribute in self.other_display_attributes:
      response['voprov:' + attribute] = self.__dict__[attribute]
    return response


################################################################################
# Define the WasGeneratedBy class mapped to the wasGeneratedBy table
class WasGeneratedBy(provBase):
  __tablename__ = 'wasGeneratedBy'
  ordered_attribute_list = ['role', 'activity_key', 'entity_key']
  other_display_attributes = ['role']

  # Internal Key
  internal_key = Column(BigInteger, primary_key=True, autoincrement=True)

  # Model attributes
  role = Column(String, nullable=True)

  # n-1 relation with Activity
  activity_key = Column(BigInteger, ForeignKey('activities.internal_key'))
  activity = relationship("Activity")
  # 0..1-1 relation with Entity
  entity_key = Column(BigInteger, ForeignKey('entities.internal_key'))
  entity = relationship("Entity", uselist=False)
  # n-1 relation with GenerationDescription
  generationDescription_key = Column(BigInteger, ForeignKey('generationDescriptions.internal_key'))
  generationDescription = relationship('GenerationDescription')

  # Print method
  def __repr__(self):
    response = ""
    for attribute in self.ordered_attribute_list:
      response += "WasGeneratedBy.%s=%s\n" % (attribute, self.__dict__[attribute])
    return response

  # Other methods
  def get_display_attributes(self):
    response = {}
    for attribute in self.other_display_attributes:
      response['voprov:' + attribute] = self.__dict__[attribute]
    return response

################################################################################
# Define the Agent class mapped to the agents table


class Agent(provBase):
  __tablename__ = 'agents'
  ordered_attribute_list = ['id', 'name', 'type', 'email', 'comment',
                            'affiliation', 'phone', 'address', 'url']

  # Internal key
  internal_key = Column(BigInteger, primary_key=True, autoincrement=True)

  # Model attributes
  id = Column(String)
  name = Column(String)
  type = Column(String)
  comment = Column(String)
  email = Column(String)
  affiliation = Column(String)
  phone = Column(String)
  address = Column(String)
  url = Column(String)

  # Print method
  def __repr__(self):
    response = ""
    for attribute in self.ordered_attribute_list:
      response += "Agent.%s=%s\n" % (attribute, self.__dict__[attribute])
    return response

################################################################################
# Define the WasAssociatedWith class mapped to the wasAssociatedWith table


class WasAssociatedWith(provBase):
  __tablename__ = 'wasAssociatedWith'
  ordered_attribute_list = ['activity_key', 'agent_key', 'role']
  other_display_attributes = ['role']

  # Internal key
  internal_key = Column(BigInteger, primary_key=True, autoincrement=True)

  # Model attributes
  role = Column(String, nullable=True)

  # n-1 relation with Activity
  activity_key = Column(BigInteger, ForeignKey('activities.internal_key'))
  activity = relationship("Activity", backref='wasAssociatedWith')
  # n-1 relation with Agent
  agent_key = Column(BigInteger, ForeignKey('agents.internal_key'))
  agent = relationship("Agent", backref='wasAssociatedWith')

  # Print method
  def __repr__(self):
    response = ""
    for attribute in self.ordered_attribute_list:
      response += "WasAssociatedWith.%s=%s\n" % (attribute, self.__dict__[attribute])
    return response

  # Other methods
  def get_display_attributes(self):
    response = {}
    for attribute in self.other_display_attributes:
      response['voprov:' + attribute] = self.__dict__[attribute]
    return response

################################################################################
# Define the WasAttributedTo class mapped to the wasAttributedTo table


class WasAttributedTo(provBase):
  __tablename__ = 'wasAttributedTo'
  ordered_attribute_list = ['entity_key', 'agent_key', 'role']
  other_display_attributes = ['role']

  # Internal key
  internal_key = Column(BigInteger, primary_key=True, autoincrement=True)

  # Model attributes
  role = Column(String, nullable=True)

  # n-1 relation with Entity
  entity_key = Column(BigInteger, ForeignKey('entities.internal_key'))
  entity = relationship("Entity", backref='wasAttributedTo')
  # n-1 relation with Agent
  agent_key = Column(BigInteger, ForeignKey('agents.internal_key'))
  agent = relationship("Agent", backref='wasAttributedTo')

  # Print method
  def __repr__(self):
    response = ""
    for attribute in self.ordered_attribute_list:
      response += "WasAttributedTo.%s=%s\n" % (attribute, self.__dict__[attribute])
    return response

  # Other methods
  def get_display_attributes(self):
    response = {}
    for attribute in self.other_display_attributes:
      response['voprov:' + attribute] = self.__dict__[attribute]
    return response

################################################################################
# Define the ActivityDescription class mapped to the activityDescriptions table


class ActivityDescription(provBase):
  __tablename__ = 'activityDescriptions'
  ordered_attribute_list = ['name', 'version', 'description', 'type', 'subtype', 'doculink']
  other_display_attributes = ['name', 'version', 'description', 'type', 'subtype', 'doculink']

  # Internal key
  internal_key = Column(BigInteger, primary_key=True, autoincrement=True)

  # Model attributes
  name = Column(String)
  version = Column(String)
  description = Column(String)
  type = Column(String)
  subtype = Column(String)
  doculink = Column(String)

  # 1-n relation with ConfigFileDescription
  configFiles = relationship("ConfigFileDescription")
  # 1-n relation with ParameterDescription
  parameters = relationship("ParameterDescription")

  # Print method
  def __repr__(self):
    response = ""
    for attribute in self.ordered_attribute_list:
      response += "ActivityDescription.%s=%s\n" % (attribute, self.__dict__[attribute])
    return response

  # Other methods
  def get_display_attributes(self):
    response = {}
    for attribute in self.other_display_attributes:
      response['voprov:' + attribute] = self.__dict__[attribute]
    return response

################################################################################
# Define the EntityDescription class mapped to the entityDescriptions table


class EntityDescription(provBase):
  __tablename__ = 'entityDescriptions'
  ordered_attribute_list = ['name', 'type', 'description', 'doculink', 'classType']
  other_display_attributes = ['name', 'type', 'description', 'doculink', 'classType']

  # Internal key
  internal_key = Column(BigInteger, primary_key=True, autoincrement=True)

  # Model attributes
  name = Column(String)
  type = Column(String)
  description = Column(String)
  doculink = Column(String)
  classType = Column(String)

  # Heritage
  __mapper_args__ = {
      'polymorphic_identity': 'entityDescription',
      'polymorphic_on': classType
  }

  # Print method
  def __repr__(self):
    response = ""
    for attribute in self.ordered_attribute_list:
      response += "EntityDescription.%s=%s\n" % (attribute, self.__dict__[attribute])
    return response

  # Other methods
  def get_display_attributes(self):
    response = {}
    for attribute in self.other_display_attributes:
      response['voprov:' + attribute] = self.__dict__[attribute]
    return response

################################################################################
# Define the ValueDescription class mapped to the valueDescriptions table


class ValueDescription(EntityDescription):
  __tablename__ = 'valueDescriptions'
  # ordered_attribute_list = EntityDescription.ordered_attribute_list + ['valueType','unit','ucd','utype']
  ordered_attribute_list = EntityDescription.ordered_attribute_list
  other_display_attributes = []

  # Internal key
  internal_key = Column(BigInteger, ForeignKey('entityDescriptions.internal_key'), primary_key=True)

  # Model attributes
  valueType = Column(String)
  unit = Column(String)
  ucd = Column(String)
  utype = Column(String)

  # Heritage
  __mapper_args__ = {'polymorphic_identity': 'valueDescription'}

  # Print method
  def __repr__(self):
    response = ""
    for attribute in self.ordered_attribute_list:
      response += "ValueDescription.%s=%s\n" % (attribute, self.__dict__[attribute])
    return response

  # Other methods
  def get_display_attributes(self):
    response = {}
    for attribute in self.other_display_attributes:
      response['voprov:' + attribute] = self.__dict__[attribute]
    return response

################################################################################
# Define the DatasetDescription class mapped to the datasetDescriptions table


class DatasetDescription(EntityDescription):
  __tablename__ = 'datasetDescriptions'
  # ordered_attribute_list = EntityDescription.ordered_attribute_list + ['contentType']
  ordered_attribute_list = EntityDescription.ordered_attribute_list
  other_display_attributes = []

  # Internal key
  internal_key = Column(BigInteger, ForeignKey('entityDescriptions.internal_key'), primary_key=True)

  # Model attributes
  contentType = Column(String)

  # Heritage
  __mapper_args__ = {'polymorphic_identity': 'datasetDescription'}

  # Print method
  def __repr__(self):
    response = ""
    for attribute in self.ordered_attribute_list:
      response += "DatasetDescription.%s=%s\n" % (attribute, self.__dict__[attribute])
    return response

  # Other methods
  def get_display_attributes(self):
    response = {}
    for attribute in self.other_display_attributes:
      response['voprov:' + attribute] = self.__dict__[attribute]
    return response

################################################################################
# Define the UsageDescription class mapped to the usageDescriptions table


class UsageDescription(provBase):
  __tablename__ = 'usageDescriptions'
  ordered_attribute_list = ['role', 'description', 'type', 'multiplicity',
                            'activityDescription_key', 'entityDescription_key']
  # Key
  internal_key = Column(BigInteger, primary_key=True, autoincrement=True)

  # Model attributes
  role = Column(String, nullable=True)
  description = Column(String)
  type = Column(String)
  multiplicity = Column(Integer)

  # n-1 relation with ActivityDescription
  activityDescription_key = Column(BigInteger, ForeignKey('activityDescriptions.internal_key'))
  activityDescription = relationship("ActivityDescription")
  # n-1 relation with EntityDescription
  entityDescription_key = Column(BigInteger, ForeignKey('entityDescriptions.internal_key'))
  entityDescription = relationship("EntityDescription")

  # Print method
  def __repr__(self):
    response = ""
    for attribute in self.ordered_attribute_list:
      response += "UsageDescription.%s=%s\n" % (attribute, self.__dict__[attribute])
    return response

  # Other methods
  def get_display_attributes(self):
    response = {}
    for attribute in self.other_display_attributes:
      response['voprov:' + attribute] = self.__dict__[attribute]
    return response

####################################################################################################
# Define the GenerationDescription class mapped to the generationDescriptions table


class GenerationDescription(provBase):
  __tablename__ = 'generationDescriptions'
  ordered_attribute_list = ['role', 'description', 'type',
                            'activityDescription_key', 'entityDescription_key']
  other_display_attributes = ['role', 'description', 'type', 'multiplicity']

  # Internal key
  internal_key = Column(BigInteger, primary_key=True, autoincrement=True)

  # Model attributes
  role = Column(String, nullable=True)
  description = Column(String)
  type = Column(String)
  multiplicity = Column(Integer)

  # Relations
  activityDescription_key = Column(BigInteger, ForeignKey('activityDescriptions.internal_key'))
  activityDescription = relationship("ActivityDescription")
  # n-1 relation with EntityDescription
  entityDescription_key = Column(BigInteger, ForeignKey('entityDescriptions.internal_key'))
  entityDescription = relationship("EntityDescription")

  # Print method
  def __repr__(self):
    response = ""
    for attribute in self.ordered_attribute_list:
      response += "GenerationDescription.%s=%s\n" % (attribute, self.__dict__[attribute])
    return response

  # Other methods
  def get_display_attributes(self):
    response = {}
    for attribute in self.other_display_attributes:
      response['voprov:' + attribute] = self.__dict__[attribute]
    return response

################################################################################
# Define the WasConfiguredBy class mapped to the wasConfiguredBy table


class WasConfiguredBy(provBase):
  __tablename__ = 'wasConfiguredBy'
  ordered_attribute_list = ['id', 'artefactType']
  other_display_attributes = ['artefactType']

  # Internal keya
  internal_key = Column(BigInteger, primary_key=True, autoincrement=True)

  # Model attributes
  artefactType = Column(String, nullable=True)

  # n-1 relation with Activity
  activity_key = Column(BigInteger, ForeignKey('activities.internal_key'))
  activity = relationship("Activity", backref='WasConfiguredBy', uselist=False)
  # 1-1 relation with ConfigFile
  configFile_key = Column(BigInteger, ForeignKey('configFiles.internal_key'))
  configFile = relationship("ConfigFile", uselist=False)
  # 1-1 relation with Parameter
  parameter_key = Column(BigInteger, ForeignKey('parameters.internal_key'))
  parameter = relationship("Parameter", uselist=False)

  # Print method
  def __repr__(self):
    response = ""
    for attribute in self.ordered_attribute_list:
      response += "WasConfiguredBy.%s=%s\n" % (attribute, self.__dict__[attribute])
    return response

  # Other methods
  def get_display_attributes(self):
    response = {}
    for attribute in self.other_display_attributes:
      response['voprov:' + attribute] = self.__dict__[attribute]
    return response


################################################################################
# Define the Parameter class mapped to the parameters table
class Parameter(provBase):
  __tablename__ = 'parameters'
  ordered_attribute_list = ['name', 'value']
  other_display_attributes = ['name', 'value']

  # Internal key
  internal_key = Column(BigInteger, primary_key=True, autoincrement=True)

  # Model attributes
  name = Column(String)
  value = Column(String)

  # n-1 relation with ParameterDescription
  parameterDescription_key = Column(BigInteger, ForeignKey("parameterDescriptions.internal_key"))
  parameterDescription = relationship("ParameterDescription")

  # Print method
  def __repr__(self):
    response = ""
    for attribute in self.ordered_attribute_list:
      response += "Parameter.%s=%s\n" % (attribute, self.__dict__[attribute])
    return response

  # Other methods
  def get_display_attributes(self):
    response = {}
    for attribute in self.other_display_attributes:
      response['voprov:' + attribute] = self.__dict__[attribute]
    return response


################################################################################
# Define the ConfigFile class mapped to the configFiles table
class ConfigFile(provBase):
  __tablename__ = 'configFiles'
  ordered_attribute_list = ['name', 'location', 'comment']
  other_display_attributes = ['name', 'location', 'comment']

  # Internal key
  internal_key = Column(BigInteger, primary_key=True, autoincrement=True)

  # Model attributes
  name = Column(String)
  location = Column(String)
  comment = Column(String)

  # n-1 relation with ConfigFileDescription
  configFileDescription_key = Column(BigInteger, ForeignKey("configFileDescriptions.internal_key"))
  configFileDescription = relationship("ConfigFileDescription")

  # Print method
  def __repr__(self):
    response = ""
    for attribute in self.ordered_attribute_list:
      response += "ConfigFile.%s=%s\n" % (attribute, self.__dict__[attribute])
    return response

  # Other methods
  def get_display_attributes(self):
    response = {}
    for attribute in self.other_display_attributes:
      response['voprov:' + attribute] = self.__dict__[attribute]
    return response


################################################################################
# Define the ParameterDescription class mapped to the parameters table
class ParameterDescription(provBase):
  __tablename__ = 'parameterDescriptions'
  ordered_attribute_list = ['name', 'valueType', 'description',
                            'unit', 'ucd', 'utype', 'min', 'max', 'default', 'options', 'activityDescription_key']
  other_display_attributes = ['name', 'valueType', 'description',
                              'unit', 'ucd', 'utype', 'min', 'max', 'default', 'options']

  # Internal key
  internal_key = Column(BigInteger, primary_key=True, autoincrement=True)

  # Model attributes
  name = Column(String)
  valueType = Column(String)
  description = Column(String)
  unit = Column(String)
  ucd = Column(String)
  utype = Column(String)
  min = Column(String)
  max = Column(String)
  default = Column(String)
  options = Column(String)

  # n-1 relation with ActivityDescription
  activityDescription_key = Column(BigInteger, ForeignKey("activityDescriptions.internal_key"))
  activityDescription = relationship("ActivityDescription")

  # Print method
  def __repr__(self):
    response = ""
    for attribute in self.ordered_attribute_list:
      response += "ParameterDescription.%s=%s\n" % (attribute, self.__dict__[attribute])
    return response

  # Other methods
  def get_display_attributes(self):
    response = {}
    for attribute in self.other_display_attributes:
      response['voprov:' + attribute] = self.__dict__[attribute]
    return response


################################################################################
# Define the ConfigFileDescription class mapped to the configFileDescriptions table
class ConfigFileDescription(provBase):
  __tablename__ = 'configFileDescriptions'
  ordered_attribute_list = ['name', 'contentType', 'description', 'activityDescription_key']
  other_display_attributes = ['name', 'contentType', 'description']

  # Internal key
  internal_key = Column(BigInteger, primary_key=True, autoincrement=True)

  # Model attributes
  name = Column(String)
  contentType = Column(String)
  description = Column(String)

  # n-1 relation with ActivityDescription
  activityDescription_key = Column(BigInteger, ForeignKey("activityDescriptions.internal_key"))
  activityDescription = relationship("ActivityDescription")

  def __repr__(self):
    response = ""
    for attribute in self.ordered_attribute_list:
      response += "ConfigFileDescription.%s=%s\n" % (attribute, self.__dict__[attribute])
    return response

  # Other methods
  def get_display_attributes(self):
    response = {}
    for attribute in self.other_display_attributes:
      response['voprov:' + attribute] = self.__dict__[attribute]
    return response


################################################################################
class ProvenanceDB(object):
  '''
    Class that defines the interactions with the tables of the ProvenanceDB.
  '''

  def __getDBConnectionInfo(self, fullname):
    """ Collect from the CS all the info needed to connect to the DB.
        This should be in a base class eventually
    """

    result = getDBParameters(fullname)
    if not result['OK']:
      raise Exception('Cannot get database parameters: %s' % result['Message'])

    dbParameters = result['Value']
    self.dbHost = dbParameters['Host']
    self.dbPort = dbParameters['Port']
    self.dbUser = dbParameters['User']
    self.dbPass = dbParameters['Password']
    self.dbName = dbParameters['DBName']

  def __init__(self):
    """c'tor
    :param self: self reference
    """

    self.log = gLogger.getSubLogger('ProvenanceDB')
    # Initialize the connection info
    self.__getDBConnectionInfo('DataManagement/ProvenanceDB')

    runDebug = (gLogger.getLevel() == 'DEBUG')
    self.engine = create_engine('postgresql://%s:%s@%s:%s/%s' % (self.dbUser,
                                                                 self.dbPass,
                                                                 self.dbHost,
                                                                 self.dbPort,
                                                                 self.dbName),
                                echo=runDebug)

    self.sessionMaker_o = sessionmaker(bind=self.engine)
    self.inspector = Inspector.from_engine(self.engine)

    # These are the list of tables that will be created.
    self.__initializeDB()

  def __initializeDB(self):
    """
    Create the tables, if they are not there yet
    """

    # sqlalchemy creates the database for me
    provBase.metadata.create_all(self.engine)

  def _sessionAdd(self, provInstance):

    session = self.sessionMaker_o()
    try:
      session.add(provInstance)
      session.commit()
      res = session.query(func.max(provInstance.internal_key).label('internal_key')).one()
      session.commit()
      return S_OK({'internal_key': res.internal_key})
    except exc.IntegrityError as err:
      self.log.warn("insert: trying to insert a duplicate key? %s" % err)
      session.rollback()
      return S_ERROR("Key already exists")
    except exc.SQLAlchemyError as e:
      session.rollback()
      self.log.exception("insert: unexpected exception", lException=e)
      return S_ERROR("insert: unexpected exception %s" % e)
    finally:
      session.close()

  def _dictToObject(self, table, fromDict):
    '''
      Add Agent
      :param agentDict:
      :return:
    '''

    fromDict = fromDict if isinstance(fromDict, dict)\
        else json.loads(fromDict) if isinstance(fromDict, six.string_types)\
        else {}

    for key, value in fromDict.items():
      # The JSON module forces the use of UTF-8, which is not properly
      # taken into account in DIRAC.
      # Not shure we also need to verify StringTypes'
      if isinstance(value, six.string_types):
        value = value.encode()

      if value:
        setattr(table, key, value)

    return table

  def addAgent(self, rowDict):
    '''
      Add Agent
      :param rowDict:
      :return:
    '''

    agent = Agent()
    row = self._dictToObject(agent, rowDict)

    try:
      res = self.getAgentKey(agent.id)['Value']['internal_key']
      return S_OK({"internal_key": res})
    except BaseException:
      return self._sessionAdd(row)

  def addActivity(self, rowDict):
    '''
      Add Activity
      :param rowDict:
      :return:
    '''

    activity = Activity()
    row = self._dictToObject(activity, rowDict)

    return self._sessionAdd(row)

  def addWasAssociatedWith(self, rowDict):
    '''
      Add WasAssociatedWith
      :param rowDict:
      :return:
    '''

    wasAssociatedWith = WasAssociatedWith()
    row = self._dictToObject(wasAssociatedWith, rowDict)

    return self._sessionAdd(row)

  def addActivityDescription(self, rowDict):
    '''
      Add ActivityDescription
      :param rowDict:
      :return:
    '''

    activityDesc = ActivityDescription()
    row = self._dictToObject(activityDesc, rowDict)
    try:
      res = self.getActivityDescriptionKey(activityDesc.name, activityDesc.version)['Value']['internal_key']
      return S_OK({"internal_key": res})
    except BaseException:
      return self._sessionAdd(row)

  def addDatasetDescription(self, rowDict):
    '''
      Add DatasetDescription
      :param rowDict:
      :return:
    '''

    datasetDesc = DatasetDescription()
    row = self._dictToObject(datasetDesc, rowDict)
    try:
      res = self.getEntityDescriptionKey(datasetDesc.name)['Value']['internal_key']
      return S_OK({"internal_key": res})
    except BaseException:
      return self._sessionAdd(row)

  def addUsageDescription(self, rowDict):
    '''
      Add UsageDescription
      :param rowDict:
      :return:
    '''

    usageDesc = UsageDescription()
    row = self._dictToObject(usageDesc, rowDict)
    try:
      res = self.getUsageDescriptionKey(usageDesc.activityDescription_key,
                                        usageDesc.entityDescription_key,
                                        usageDesc.role)['Value']['internal_key']
      return S_OK({"internal_key": res})
    except BaseException:
      return self._sessionAdd(row)

  def addGenerationDescription(self, rowDict):
    '''
      Add UsageDescription
      :param rowDict:
      :return:
    '''

    generationDesc = GenerationDescription()
    row = self._dictToObject(generationDesc, rowDict)
    try:
      res = self.getGenerationDescriptionKey(generationDesc.activityDescription_key,
                                             generationDesc.entityDescription_key,
                                             generationDesc.role)['Value']['internal_key']
      return S_OK({"internal_key": res})
    except BaseException:
      return self._sessionAdd(row)

  def addDatasetEntity(self, rowDict):
    '''
      Add DatasetEntity
      :param rowDict:
      :return:
    '''

    datasetEntity = DatasetEntity()
    row = self._dictToObject(datasetEntity, rowDict)
    return self._sessionAdd(row)

  def addWasAttributedTo(self, rowDict):
    '''
      Add WasAttributedTo
      :param rowDict:
      :return:
    '''

    wasAttributedTo = WasAttributedTo()
    row = self._dictToObject(wasAttributedTo, rowDict)
    return self._sessionAdd(row)

  def addUsed(self, rowDict):
    '''
      Add Used
      :param rowDict:
      :return:
    '''

    used = Used()
    row = self._dictToObject(used, rowDict)
    return self._sessionAdd(row)

  def addWasGeneratedBy(self, rowDict):
    '''
      Add WasGeneratedBy
      :param rowDict:
      :return:
    '''

    wasGeneratedBy = WasGeneratedBy()
    row = self._dictToObject(wasGeneratedBy, rowDict)
    return self._sessionAdd(row)

  def addValueEntity(self, rowDict):
    '''
      Add ValueEntity
      :param rowDict:
      :return:
    '''

    valueEntity = ValueEntity()
    row = self._dictToObject(valueEntity, rowDict)
    return self._sessionAdd(row)

  def addValueDescription(self, rowDict):
    '''
      Add ValueDescription
      :param rowDict:
      :return:
    '''

    valueDesc = ValueDescription()
    row = self._dictToObject(valueDesc, rowDict)
    try:
      res = self.getEntityDescriptionKey(valueDesc.name)['Value']['internal_key']
      return S_OK({"internal_key": res})
    except BaseException:
      return self._sessionAdd(row)

  def addWasConfiguredBy(self, rowDict):
    '''
      Add WasConfiguredBy
      :param rowDict:
      :return:
    '''

    wasConfiguredBy = WasConfiguredBy()
    row = self._dictToObject(wasConfiguredBy, rowDict)
    return self._sessionAdd(row)

  def addParameter(self, rowDict):
    '''
      Add Parameter
      :param rowDict:
      :return:
    '''

    parameter = Parameter()
    row = self._dictToObject(parameter, rowDict)
    return self._sessionAdd(row)

  def addConfigFile(self, rowDict):
    '''
      Add ConfigFile
      :param rowDict:
      :return:
    '''

    configFile = ConfigFile()
    row = self._dictToObject(configFile, rowDict)
    return self._sessionAdd(row)

  def addParameterDescription(self, rowDict):
    '''
      Add ParameterDescription
      :param rowDict:
      :return:
    '''

    parameterDescription = ParameterDescription()
    row = self._dictToObject(parameterDescription, rowDict)
    return self._sessionAdd(row)

  def addConfigFileDescription(self, rowDict):
    '''
      Add ConfigFileDescription
      :param rowDict:
      :return:
    '''

    configFileDescription = ConfigFileDescription()
    row = self._dictToObject(configFileDescription, rowDict)
    return self._sessionAdd(row)

  def getAgents(self):
    '''
      Get Agents
      :return:
    '''

    session = self.sessionMaker_o()
    agentIDs = []
    try:
      for instance in session.query(Agent):
        agentIDs.append(instance.internal_key)
      session.commit()
      return S_OK(agentIDs)
    except NoResultFound as e:
      return S_OK()
    finally:
      session.close()

  def getDatasetEntity(self, guid):
    '''
      Get DatasetEntity
      :param guid
      :return:
    '''

    session = self.sessionMaker_o()
    try:
      datasetEntity = session.query(DatasetEntity)\
          .filter(DatasetEntity.id == guid) \
          .filter(DatasetEntity.invalidatedAtTime is None) \
          .one()
      return S_OK(datasetEntity.internal_key)
    except NoResultFound as e:
      return S_OK()
    finally:
      session.close()

  def updateDatasetEntity(self, internal_key, invalidatedAtTime):
    '''
      Update DatasetEntity
      :param internal_key, invalidatedAtTime
      :return:
    '''

    session = self.sessionMaker_o()
    try:
      datasetEntity = session.update(DatasetEntity)\
          .where(DatasetEntity.internal_key == internal_key)\
          .values(invalidatedAtTime=invalidatedAtTime)
      session.commit()
      return S_OK()
    except e:
      return S_OK()
    finally:
      session.close()

  def getUsageDescription(self, activityDescription_key, role):
    '''
      Get UsageDescription
      :param activityDescription_key, role
      :return: usageDescription.internal_key
    '''

    session = self.sessionMaker_o()
    try:
      usageDescription = session.query(UsageDescription)\
          .filter(UsageDescription.activityDescription_key == activityDescription_key, UsageDescription.role == role)\
          .one()
      session.commit()
      return S_OK({'internal_key': usageDescription.internal_key,
                   'entityDescription_key': usageDescription.entityDescription_key})
    except NoResultFound as e:
      return S_OK()
    finally:
      session.close()

  def getGenerationDescription(self, activityDescription_key, role):
    '''
      Get GenerationDescription
      :param activityDescription_key, role
      :return: generationDescription.internal_key
    '''

    session = self.sessionMaker_o()
    try:
      generationDescription = session.query(GenerationDescription)\
          .filter(GenerationDescription.activityDescription_key == activityDescription_key)\
          .filter(GenerationDescription.role == role)\
          .one()
      session.commit()
      return S_OK({'internal_key': generationDescription.internal_key,
                   'entityDescription_key': generationDescription.entityDescription_key})
    except NoResultFound as e:
      return S_OK()
    finally:
      session.close()

  def getParameterDescription(self, activityDescription_key, parameter_name):
    '''
      Get getParameterDescription
      :param activityDescription_key, parameter_name
      :return: parameterDescription.internal_key
    '''

    session = self.sessionMaker_o()
    try:
      parameterDescription = session.query(ParameterDescription)\
          .filter(ParameterDescription.activityDescription_key == activityDescription_key,
                  ParameterDescription.name == parameter_name)\
          .one()
      session.commit()
      return S_OK({'internal_key': parameterDescription.internal_key})
    except NoResultFound as e:
      return S_OK()
    finally:
      session.close()

  def getConfigFileDescription(self, activityDescription_key, configFile_name):
    '''
      Get getConfigFileDescription
      :param activityDescription_key, configFile_name
      :return: configFileDescription.internal_key
    '''

    session = self.sessionMaker_o()
    try:
      configFileDescription = session.query(ConfigFileDescription)\
          .filter(ConfigFileDescription.activityDescription_key == activityDescription_key,
                  ConfigFileDescription.name == configFile_name)\
          .one()
      session.commit()
      return S_OK({'internal_key': configFileDescription.s})
    except NoResultFound as e:
      return S_OK()
    finally:
      session.close()

  def getActivityDescriptionKey(self, activityDescription_name, activityDescription_version):
    """
      Get getActivityDescriptionKey
      :param activityDescription_name, activityDescription_version
      :return: ActivityDescription.internal_key
    """
    session = self.sessionMaker_o()
    try:
      activityDescription_list = session.query(ActivityDescription) \
          .filter(ActivityDescription.name == activityDescription_name) \
          .filter(ActivityDescription.version == activityDescription_version) \
          .all()
      activityDescription_last = activityDescription_list[-1]
      return S_OK({'internal_key': activityDescription_last.internal_key})

    except NoResultFound as e:
      return S_OK()
    except BaseException:
      return S_OK()
    finally:
      session.close()

  def getEntityDescriptionKey(self, entityDescription_name):
    """
      Get getEntityDescriptionKey
      :param entityDescription_name
      :return: EntityDescription.internal_key
    """
    session = self.sessionMaker_o()
    try:
      entityDescription_list = session.query(EntityDescription)\
          .filter(EntityDescription.name == entityDescription_name)\
          .all()

      entityDescription_last = entityDescription_list[-1]
      return S_OK({'internal_key': entityDescription_last.internal_key})
    except NoResultFound as e:
      return S_OK()
    finally:
      session.close()

  def getUsageDescriptionKey(self, usageDescription_activity, usageDescription_entity, usageDescription_role):
    """
      Get getUsageDescriptionKey
      :param usageDescription_activity, usageDescription_entity, usageDescription_role
      :return: UsageDescription.internal_key
    """
    session = self.sessionMaker_o()
    try:
      usageDescription_list = session.query(UsageDescription) \
          .filter(UsageDescription.activityDescription_key == usageDescription_activity) \
          .filter(UsageDescription.entityDescription_key == usageDescription_entity) \
          .filter(UsageDescription.role == usageDescription_role) \
          .all()

      usageDescription_last = usageDescription_list[-1]
      return S_OK({'internal_key': usageDescription_last.internal_key})
    except NoResultFound as e:
      return S_OK()
    finally:
      session.close()

  def getGenerationDescriptionKey(
          self,
          generationDescription_activity,
          generationDescription_entity,
          generationDescription_role):
    """
      Get getGenerationDescriptionKey
      :param generationDescription_activity, generationDescription_entity, generationDescription_role
      :return: GenerationDescription.internal_key
    """
    session = self.sessionMaker_o()
    try:
      generationDescription_list = session.query(GenerationDescription) \
          .filter(GenerationDescription.activityDescription_key == generationDescription_activity) \
          .filter(GenerationDescription.entityDescription_key == generationDescription_entity) \
          .filter(GenerationDescription.role == generationDescription_role) \
          .all()

      generationDescription_last = generationDescription_list[-1]
      return S_OK({'internal_key': generationDescription_last.internal_key})
    except NoResultFound as e:
      return S_OK()
    finally:
      session.close()

  def getAgentKey(self, agent_id):
    """
      Get AgentKey
      :param agent_id
      :return: Agent.internal_key
    """
    session = self.sessionMaker_o()
    try:
      agent_list = session.query(Agent) \
          .filter(Agent.id == agent_id) \
          .all()
      agent_last = agent_list[-1]
      return S_OK({'internal_key': agent_last.internal_key})
    except NoResultFound as e:
      return S_OK()
    finally:
      session.close()



#!/usr/bin/env python

"""
  Get the files attached to a transformation and optionally update the file status
  Extension of dirac-transformation-get-files with additional options:

  Usage:
    cta-transformation-get-files <transID or ascii file with a list of transID comma separated> <options>

  Options:
  --FileStatus : selection based on the FileStatus
  --TaskStatus : selection based on the TaskStatus
  --setFileStatus : update the FileStatus of the selected files to a new Status
  --dumpLFN : dump the list of selected LFNs in a file
  --dumpFull : dump the list of selected <TransID,LFN> in a file
"""

__RCSID__ = "$Id$"

import os

import DIRAC
from DIRAC import gLogger
from DIRAC.Core.Utilities.DIRACScript import DIRACScript as Script
from DIRAC.TransformationSystem.Client.TransformationClient import TransformationClient

@Script()
def main():

  Script.registerSwitch('', 'FileStatus=', '    file status')
  Script.registerSwitch('', 'TaskStatus=', '    task status')
  Script.registerSwitch('', 'setFileStatus=', '    new file status')
  Script.registerSwitch('', 'dumpLFN', '    dump the list of selected LFNs in a file')
  Script.registerSwitch('', 'dumpFull', '    dump the list of selected <TransID,LFN> in a file')
  switches, argss = Script.parseCommandLine(ignoreErrors=True)

  transIDList = []
  fileStatus = None
  taskStatus = None
  newFileStatus = None
  dumpLFNFlag = False
  dumpFullFlag = False

  for switch in switches:
    if switch[0] == 'FileStatus':
      fileStatus = switch[1].capitalize()
    elif switch[0] == 'TaskStatus':
      taskStatus = switch[1].capitalize()
    elif switch[0] == 'setFileStatus':
      newFileStatus = switch[1].capitalize()
    elif switch[0] == 'dumpLFN':
      dumpLFNFlag = True
    elif switch[0] == 'dumpFull':
      dumpFullFlag = True

  if len(argss)!=1:
    Script.showHelp()
    DIRAC.exit(-1)

  if os.path.isfile(argss[0]):
    gLogger.notice('\nReading transformation IDs from input file: %s\n' % argss[0])
    lines = open(argss[0], 'rt').readlines()
    transIDList += [transID.strip() for line in lines for transID in line.split(',')]
    gLogger.notice("Found %d transformations" % len(transIDList))
  else:
    transIDList.append(argss[0])

  if fileStatus or taskStatus:
    outputFileName = ("trans%sFiles_%sTasks_%s-%s" % (fileStatus,taskStatus,transIDList[0],transIDList[-1]))

  if (dumpLFNFlag):
    outputLFN = outputFileName + '.lfns'
    f0 = open(outputLFN, 'w')
    
  if (dumpFullFlag):
    outputFull = outputFileName + '.transID_lfns'
    f1 = open(outputFull, 'w')

  tc = TransformationClient()

  for transID in transIDList:

    condDict = {'TransformationID': transID}

    if fileStatus:
      condDict.update({'Status': fileStatus})

    if taskStatus:
      res = tc.getTransformationTasks({'TransformationID': transID, 'ExternalStatus': taskStatus})

      if not res['OK']:
        gLogger.error(res['Message'])
        DIRAC.exit(2)

      if len(res['Value']) == 0:
        gLogger.notice("No tasks selected for transformation %s with status %s" % (transID, taskStatus))

      taskIDs = []
      for task in res['Value']:
        taskIDs.append(task["TaskID"])

      condDict.update({'TaskID': taskIDs})

    res = tc.getTransformationFiles(condDict)

    if not res['OK']:
      gLogger.error(res['Message'])
      DIRAC.exit(2)

    if len(res['Value']) == 0:
      gLogger.notice("No files selected for transformation %s with status %s" % (transID, fileStatus))

    transFiles = []
    for transfile in res['Value']:
      transFiles.append(transfile['LFN'])
      if (dumpLFNFlag):
        f0.write("%s\n" % (transfile['LFN']))
      if (dumpFullFlag):
        f1.write("%s %s\n" % (transID, transfile['LFN']))
      else:
        gLogger.notice(transfile['LFN'])

    if newFileStatus:
      res = tc.setFileStatusForTransformation(transID, newLFNsStatus=newFileStatus, lfns=transFiles)
      if not res['OK']:
        gLogger.error(res['Message'])
        DIRAC.exit(2)
      gLogger.notice(" %d files set to %s status" % (len(transFiles), newFileStatus))

  if (dumpLFNFlag):
    gLogger.notice('%d files dumped in %s' % (len(transFiles), outputLFN))

  if (dumpFullFlag):
    gLogger.notice('%d files dumped in %s' % (len(transFiles), outputFull))

if __name__ == "__main__":
  main()



#!/usr/bin/env python

__RCSID__ = "$Id$"

# generic imports
import os
import six

# DIRAC imports
from DIRAC.Core.Utilities.DIRACScript import DIRACScript as Script
import DIRAC

Script.parseCommandLine(ignoreErrors=True)

from DIRAC import gLogger
from DIRAC.Interfaces.API.Dirac import Dirac
from DIRAC.Resources.Catalog.FileCatalogClient import FileCatalogClient

@Script()
def main():

  fcc = FileCatalogClient()
  dirac = Dirac()
  resJDL = dirac.getJobJDL(os.environ['JOBID'])

  idata = resJDL['Value']['InputData']
  if isinstance(idata, six.string_types):
    idata = []
    if 'LFN' in resJDL['Value']['InputData']:
      idata.append(resJDL['Value']['InputData'].split('LFN:')[1])
    else:
      idata.append(resJDL['Value']['InputData'])
  else:
    idata = resJDL['Value']['InputData']

  for lfn in idata:
    gLogger.notice("Checking input file:\n %s " % lfn)
    file_name = os.path.basename(lfn)
    code = os.system("./dirac_simtel_check " + file_name)
    if code!=0:
      gLogger.error("Corrupted file:\n %s" % lfn)
      gLogger.notice("Setting nsb=-1")
      res = fcc.setMetadata(lfn,{'nsb': -1})
      if not res['OK']:
        return res

  DIRAC.exit()

if __name__ == '__main__':
  main()



"""
Launcher script to launch Prod5CtaPipeModelingJob on the WMS or create a Transformation

Usage:
python prod5_ctapipe_modeling_launcher.py TS <trans_prefix> <dataset name or ascii file with a list of datasets> <group_size>

In testing mode (WMS):
python prod5_ctapipe_modeling_launcher.py WMS

Arguments:
mode: WMS for testing, TS for production
Arguments with TS mode:
trans_prefix: prefix to add to the name of the transformation
input_dataset: name of the input dataset
group_size: n files to process

Example:
python prod5_ctapipe_modeling_launcher.py TS ctapipe_modeling Prod5b_LaPalma_AdvancedBaseline_NSB1x_electron_North_40deg_DL0 5
"""
import os
from copy import copy

from DIRAC.Core.Base import Script

Script.parseCommandLine()

import DIRAC
from CTADIRAC.Core.Utilities.tool_box import read_inputs_from_file
from DIRAC.TransformationSystem.Client.Transformation import Transformation
from CTADIRAC.Interfaces.API.Prod5CtaPipeModelingJob import Prod5CtaPipeModelingJob
from DIRAC.Interfaces.API.Dirac import Dirac
from CTADIRAC.Core.Utilities.tool_box import get_dataset_MQ


def submit_trans(job, trans_name, input_meta_query, group_size):
    """ Create a transformation executing the job workflow
  """
    DIRAC.gLogger.notice('submit_trans : %s' % trans_name)

    trans = Transformation()
    trans.setTransformationName(trans_name)  # this must be unique
    trans.setType("DataReprocessing")
    trans.setDescription("ctapipe Modeling TS")
    trans.setLongDescription("ctapipe Modeling processing")  # mandatory
    trans.setBody(job.workflow.toXML())
    trans.setGroupSize(group_size)
    trans.setInputMetaQuery(input_meta_query)
    res = trans.addTransformation()  # transformation is created here
    if not res['OK']:
        return res
    trans.setStatus("Active")
    trans.setAgentType("Automatic")
    trans_id = trans.getTransformationID()
    return trans_id


def submit_wms(job):
    """ Submit the job to the WMS
  @todo launch job locally
  """
    dirac = Dirac()
    input_data = '''
/vo.cta.in2p3.fr/MC/PROD5/Paranal/gamma/sim_telarray/2224/Data/000xxx/gamma_20deg_0deg_run102___cta-prod5-paranal_desert-2147m-Paranal-dark.simtel.zst
/vo.cta.in2p3.fr/MC/PROD5/Paranal/gamma/sim_telarray/2224/Data/000xxx/gamma_20deg_0deg_run103___cta-prod5-paranal_desert-2147m-Paranal-dark.simtel.zst
/vo.cta.in2p3.fr/MC/PROD5/Paranal/gamma/sim_telarray/2224/Data/000xxx/gamma_20deg_0deg_run104___cta-prod5-paranal_desert-2147m-Paranal-dark.simtel.zst
    '''.split()

    job.setInputData(input_data)
    job.setJobGroup('prod5_ctapipe_modeling')
    res = dirac.submitJob(job)
    if res['OK']:
        Script.gLogger.notice('Submitted job: ', res['Value'])
    return res

def launch_job(args):
    """ Simple launcher to instanciate a Job and setup parameters
      from positional arguments given on the command line.

      Parameters:
      args -- mode (trans_name dataset_name group_size)
  """
    DIRAC.gLogger.notice('Launching Modeling processing')
    # get arguments
    mode = args[0]

    if mode == 'TS':
        name_prefix = args[1]
        group_size = int(args[3])

        dataset_list = []
        if os.path.isfile(args[2]):
            DIRAC.gLogger.notice('Reading datasets from input file:', args[2])
            dataset_list = read_inputs_from_file(args[2])
        else:
            dataset_list.append(args[2])

    # use this mode for testing
    if mode == 'WMS':
        # job setup
        job = Prod5CtaPipeModelingJob(cpuTime=259200.)
        #  job.ctapipe_site_config = 'prod5b_lapalma_alpha.yml'
        job.ctapipe_site_config = 'prod5b_paranal_alpha_nectarcam.yml'
        # override for testing
        job.setName('Prod5_ctapipe_modeling')
        # output
        job.setOutputSandbox(['*Log.txt'])
        job.base_path = '/vo.cta.in2p3.fr/user/a/arrabito'
        # just to test the metadata functionality
        # the following metadata values are arbitrary
        output_meta_data = {'array_layout': 'Advanced-Baseline', 'site': 'Paranal',
                            'particle': 'gamma-diffuse', 'phiP': 180.0, 'thetaP': 20.0}
        job.set_meta_data(output_meta_data)
        job.set_file_meta_data(nsb=1)
        job.setupWorkflow(debug=True)
        res = submit_wms(job)

    # use this mode for production
    elif mode == 'TS':
        for dataset_name in dataset_list:
            # job setup
            job = Prod5CtaPipeModelingJob(cpuTime=259200.)
            job.setName('Prod5_ctapipe_modeling')
            job.base_path = '/vo.cta.in2p3.fr/MC/PROD5'
            job.setOutputSandbox(['*Log.txt'])
            # set the metadata
            input_meta_query = get_dataset_MQ(dataset_name)
            output_meta_data = copy(input_meta_query)
            job.set_meta_data(output_meta_data)
            job.set_file_meta_data(nsb=output_meta_data['nsb']['='],split=output_meta_data['split'])
            
            # set site dependent config
            cta_site = input_meta_query['site'].lower()
            if cta_site == 'paranal':
                job.ctapipe_site_config = 'prod5b_paranal_alpha_nectarcam.yml'
            elif cta_site == 'lapalma':
                job.ctapipe_site_config = 'prod5b_lapalma_alpha.yml'

            # define the job steps
            job.setupWorkflow(debug=False)
            job.setType('Stage1Processing')  # mandatory *here*

            # submit the transformations
            trans_name = name_prefix + '_' + dataset_name
            res = submit_trans(job, trans_name, input_meta_query, group_size)

    else:
        DIRAC.gLogger.error('1st argument should be the job mode: WMS or TS,\n\
                             not %s' % mode)
        return None

    return res


#########################################################
if __name__ == '__main__':

    arguments = Script.getPositionalArgs()
    if len(arguments) not in [1, 4]:
        Script.showHelp()
    try:
        res = launch_job(arguments)
        if not res['OK']:
            DIRAC.gLogger.error(res['Message'])
            DIRAC.exit(-1)
        else:
            DIRAC.gLogger.notice('Done')
    except Exception:
        DIRAC.gLogger.exception()
        DIRAC.exit(-1)



#!/usr/bin/env python
"""
  Check if files exist in the Catalog
  and dump the list of Found and Not Found files

  Usage:
  cta-dms-check-lfn [options] <ascii file with lfn list>
"""

__RCSID__ = "$Id$"

import DIRAC
from DIRAC.Core.Utilities.DIRACScript import DIRACScript as Script
from DIRAC import gLogger
from DIRAC.Resources.Catalog.FileCatalogClient import FileCatalogClient
from CTADIRAC.Core.Utilities.tool_box import read_inputs_from_file

@Script()
def main():
  Script.parseCommandLine(ignoreErrors=True)
  args = Script.getPositionalArgs()
  if len(args) > 0:
    infile = args[0]
  else:
    Script.showHelp()

  lfns = read_inputs_from_file(infile)
  lfnsFound = []
  lfnsNotFound = []

  fc = FileCatalogClient()

  for lfn in lfns:
    res = fc.exists(lfn)
    if not res['OK']:
      gLogger.error(res['Message'])

    if res['Value']['Successful'][lfn]:
      gLogger.debug('%s found in Catalog' % lfn)
      lfnsFound.append(lfn)
    else:
      gLogger.debug('%s not found in Catalog' % lfn)
      lfnsNotFound.append(lfn)

  fname = infile + '_Found.list'
  f = open(fname, 'w')
  for lfn in lfnsFound:
    f.write(lfn + '\n')
  f.close()
  gLogger.notice('%d files have been dumped in %s' % (len(lfnsFound), fname))

  fname = infile + '_NotFound.list'
  f = open(fname, 'w')
  for lfn in lfnsNotFound:
    f.write(lfn + '\n')
  f.close()
  gLogger.notice('%d files have been dumped in %s' % (len(lfnsNotFound), fname))
  DIRAC.exit()

####################################################
if __name__ == '__main__':
  main()


""" Simtel Script to create a Transformation with Corsika Input Data

    https://forge.in2p3.fr/issues/33932
                        July 13th 2018 - J. Bregeon, C. Bigongiari, L. Arrabito
"""

__RCSID__ = "$Id$"

from DIRAC.Core.Base import Script
Script.setUsageMessage('\n'.join([__doc__.split('\n')[1],
                                  'Usage:',
                                  '  %s mode file_path (trans_name) group_size' % Script.scriptName,
                                  'Arguments:',
                                  '  mode: WMS for testing, TS for production',
                                  '  file_path: path to the input file with the list of LFNs to process',
                                  '  trans_name: name of the transformation for TS mode only',
                                  '  group_size: n files to process',
                                  '\ne.g: python %s.py WMS p20.txt simtel_astri_p20 10' % Script.scriptName,
                                  ]))

Script.parseCommandLine()

import DIRAC
from DIRAC.TransformationSystem.Client.TransformationClient import TransformationClient
from DIRAC.TransformationSystem.Client.Transformation import Transformation
from CTADIRAC.Interfaces.API.SimtelTSJob import SimtelTSJob
from DIRAC.Interfaces.API.Dirac import Dirac
from DIRAC.Core.Workflow.Parameter import Parameter
from CTADIRAC.Core.Utilities.tool_box import read_inputs_from_file

def submit_trans(job, infileList, trans_name, group_size):
  """ Create a transformation executing the job workflow
  """
  DIRAC.gLogger.notice('submit_trans : %s' % trans_name)

  # Initialize JOB_ID
  job.workflow.addParameter(Parameter("JOB_ID", "000000", "string", "", "",
                                      True, False, "Temporary fix"))

  trans = Transformation()
  trans.setTransformationName(trans_name)  # this must be unique
  trans.setType("DataReprocessing")
  trans.setDescription("Simtel TS example")
  trans.setLongDescription("Simtel tel_sim")  # mandatory
  trans.setBody(job.workflow.toXML())
  trans.setGroupSize(group_size)
  res = trans.addTransformation()  # transformation is created here
  if not res['OK']:
    return res
  trans.setStatus("Active")
  trans.setAgentType("Automatic")
  # add 10*group_size files to transformation (to have the first 10 jobs)
  trans_id = trans.getTransformationID()
  trans_client = TransformationClient()
  res = trans_client.addFilesToTransformation(trans_id['Value'],
                                              infileList[:10 * group_size])
  return res


def submit_WMS(job, infileList):
  """ Submit the job locally or to the WMS
  """
  dirac = Dirac()
  job.setInputData(infileList)
  job.setJobGroup('SimtelJob')
  res = dirac.submitJob(job)
  if res['OK']:
    Script.gLogger.info('Submission Result: ', res['Value'])
  return res


def run_simtel_ts(args=None):
  """ Simple wrapper to create a SimtelTSJob and setup parameters
      from positional arguments given on the command line.

      Parameters:
      args -- mode file_path (trans_name) group_size
  """
  DIRAC.gLogger.notice('run_simtel_ts')
  # get arguments
  mode = args[0]
  if mode == 'WMS':
    file_path = args[1]
    group_size = int(args[2])
  elif mode == 'TS':
    file_path = args[1]
    trans_name = args[2]
    group_size = int(args[3])
  else:
    DIRAC.gLogger.error('1st argument should be the job mode: WMS or TS,\n\
                             not %s' % args[0])
    DIRAC.exit(-1)

  # read list of input file names
  input_file_list = read_inputs_from_file(file_path)

  ################################
  job = SimtelTSJob(cpuTime=43200)  # to be adjusted!!

  ### Main Script ###
  # override for testing
  job.setName('Simtel')
  # Allow job meshing with ByJobType plugin
  job.setType('SimtelProcessing')

  # Defaults to override
  # job.version = '2018-06-12'
  # job.simtel_config_file = 'ASTRI_MiniArray15_Paranal_ACDC_2018_06_12.cfg'
  # job.thetaP = 20.0
  # job.phiP = 0.0

  # output
  job.setOutputSandbox(['*Log.txt'])
  # Customize this path to point to your user area
  job.base_path = '/vo.cta.in2p3.fr/user/a/arrabito/Miniarray15'

  # specific configuration
  if mode == 'WMS':
    # set meta data
    # job.set_metadata(input_file_list[0])
    job.ts_task_id = '0'
    job.setupWorkflow(debug=True)
    # subtmit to the WMS for debug
    job.setDestination('LCG.IN2P3-CC.fr')
    res = submit_WMS(job, input_file_list[:group_size])
  elif mode == 'TS':
    job.ts_task_id = '@{JOB_ID}'  # dynamic
    job.setupWorkflow(debug=True)
    res = submit_trans(job, input_file_list, trans_name, group_size)
  else:
    DIRAC.gLogger.error('Unknown mode')
    return None

  return res


#########################################################
if __name__ == '__main__':

  args = Script.getPositionalArgs()
  if len(args) not in [3, 4]:
    Script.showHelp()
  try:
    res = run_simtel_ts(args)
    if not res['OK']:
      DIRAC.gLogger.error(res['Message'])
      DIRAC.exit(-1)
    else:
      DIRAC.gLogger.notice('Done')
  except Exception:
    DIRAC.gLogger.exception()
    DIRAC.exit(-1)



""" Launcher script to launch a Prod5MCPipeNSBJob
on the WMS or create a Transformation.

    https://forge.in2p3.fr/issues/38939
                    JB, June 2020
"""

import json
from copy import copy

from DIRAC.Core.Base import Script
Script.setUsageMessage('\n'.join([__doc__.split('\n')[1],
                                  'Usage:',
                                  'python %s.py <mode> <site> <particle> <pointing> <zenith> <n showers>' % Script.scriptName,
                                  'Arguments:',
                                  '  mode: WMS for testing, TS for production',
                                  '  site: either Paranal or LaPalma',
                                  '  particle: in gamma, gamma-diffuse, electron, proton',
                                  '  pointing: North or South',
                                  '  zenith: 20, 40 or 60',
                                  '  n shower: 100 for testing',
                                  '\ne.g: python %s.py WMS Paranal gamma North 20 100' % Script.scriptName,
                                  ]))

Script.parseCommandLine()

import DIRAC
from DIRAC.TransformationSystem.Client.Transformation import Transformation
from CTADIRAC.Interfaces.API.Prod5bMCPipeNSBJob import Prod5bMCPipeNSBJob
from DIRAC.Core.Workflow.Parameter import Parameter
from DIRAC.Interfaces.API.Dirac import Dirac


def submit_transformation(job, trans_name):
  """ Create a transformation executing the job workflow
  """
  DIRAC.gLogger.notice('submit_trans : %s' % trans_name)

  # Initialize JOB_ID
  job.workflow.addParameter(Parameter("JOB_ID", "000000", "string", "", "",
                                      True, False, "Temporary fix"))

  trans = Transformation()
  trans.setTransformationName(trans_name)  # this must be unique
  trans.setType("MCSimulation")
  trans.setDescription("Prod5 MC Pipe NSB TS")
  trans.setLongDescription("Prod5 simulation pipeline")  # mandatory
  trans.setBody(job.workflow.toXML())
  result = trans.addTransformation()  # transformation is created here
  if not result['OK']:
    return result
  trans.setStatus("Active")
  trans.setAgentType("Automatic")
  trans_id = trans.getTransformationID()
  return trans_id


def submit_wms(job):
  """ Submit the job to the WMS
  @todo launch job locally
  """
  dirac = Dirac()
  job.setJobGroup('Prod5MCPipeNSBJob')
  result = dirac.submitJob(job)
  if result['OK']:
    Script.gLogger.notice('Submitted job: ', result['Value'])
  return result


def run_simulation(args):
  """ Simple wrapper to create a Prod5MCPipeNSBJob and setup parameters
      from positional arguments given on the command line.

      Parameters:
      args -- mode (trans_name)
  """
  DIRAC.gLogger.notice('run_mc_pipeline')
  # get arguments
  mode = args[0]

  # job setup
  job = Prod5bMCPipeNSBJob()  # to be adjusted!!
  job.version = '2020-06-29b'
  job.compiler = 'gcc83_matchcpu'
  # override for testing
  job.setName('Prod5b_MC_Pipeline_NSB')
  # parameters from command line
  job.set_site(args[1])
  job.set_particle(args[2])
  job.set_pointing_dir(args[3])
  job.zenith_angle = args[4]
  job.n_shower = args[5]

  # output
  job.setOutputSandbox(['*Log.txt'])

  # specific configuration
  if mode == 'WMS':
    # put here your user directory under the Dirac File Catalog
    job.base_path = '/vo.cta.in2p3.fr/user/b/bregeon/prod5b_test_1/'
    # adjust start_run_number and run_number for testing, both can be 0
    job.start_run_number = '0'
    job.run_number = '11'
    job.setupWorkflow(debug=True)
    # submit to the WMS for debug, choose a destination site
    # job.setDestination('LCG.IN2P3-CC.fr')
    job.setDestination('LCG.DESY-ZEUTHEN.de')
    result = submit_wms(job)
  elif mode == 'TS':
    # put here your user directory under the Dirac File Catalog
    job.base_path = '/vo.cta.in2p3.fr/user/b/bregeon/prod5b_JB_test_3'
    job.start_run_number = '0'
    job.run_number = '@{JOB_ID}'  # dynamic
    job.setupWorkflow(debug=False)
    # extra tag in case you have to run different tests, can be empty
    tag = ''
    # Change below the name of transformation, in particular the user name
    # this name must be unique accross the whole system
    trans_name = 'bregeon_Prod5b_Test_%s_%s_%s_%s%s' %\
        (job.cta_site, job.particle, job.pointing_dir, job.zenith_angle, tag)
    result = submit_transformation(job, trans_name)
  else:
    DIRAC.gLogger.error('1st argument should be the job mode: WMS or TS,\n\
                             not %s' % mode)
    return None

  return result


#########################################################
if __name__ == '__main__':

  arguments = Script.getPositionalArgs()
  if len(arguments) != 6:
    Script.showHelp()
  try:
    result = run_simulation(arguments)
    if not result['OK']:
      DIRAC.gLogger.error(result['Message'])
      DIRAC.exit(-1)
    else:
      DIRAC.gLogger.notice('Done')
  except Exception:
    DIRAC.gLogger.exception()
    DIRAC.exit(-1)



""" 
Launcher script to launch EvnDispProd5SingJob on the WMS or create a Transformation
https://forge.in2p3.fr/issues/441709
https://forge.in2p3.fr/issues/49828

Usage:
python prod5b_sing_evndisp_launcher.py TS <trans_prefix> <dataset name or ascii file with a list of datasets> <group_size>

In testing mode (WMS):
python prod5b_sing_evndisp_launcher.py WMS

Arguments:
mode: WMS for testing, TS for production
Arguments with TS mode:
trans_prefix: prefix to add to the name of the transformation
input_dataset: name of the input dataset
group_size: n files to process

Example:
python prod5b_sing_evndisp_launcher.py TS EvnDispRepro Prod5b_LaPalma_AdvancedBaseline_NSB1x_electron_North_40deg_DL0 5
"""
import os
from copy import copy

from DIRAC.Core.Base import Script
Script.parseCommandLine()

import DIRAC
from CTADIRAC.Core.Utilities.tool_box import read_inputs_from_file
from DIRAC.TransformationSystem.Client.Transformation import Transformation
from CTADIRAC.Interfaces.API.EvnDispProd5SingJob import EvnDispProd5SingJob
from DIRAC.Core.Workflow.Parameter import Parameter
from DIRAC.Interfaces.API.Dirac import Dirac
from CTADIRAC.Core.Utilities.tool_box import get_dataset_MQ

def submit_trans(job, trans_name, input_meta_query, group_size):
  """ Create a transformation executing the job workflow
  """
  DIRAC.gLogger.notice('submit_trans : %s' % trans_name)

  # Initialize JOB_ID
  job.workflow.addParameter(Parameter("JOB_ID", "000000", "string", "", "",
                                      True, False, "Temporary fix"))

  trans = Transformation()
  trans.setTransformationName(trans_name)  # this must be unique
  trans.setType("DataReprocessing")
  trans.setDescription("Prod5 EventDisplay TS")
  trans.setLongDescription("Prod5 EventDisplay processing")  # mandatory
  trans.setBody(job.workflow.toXML())
  trans.setGroupSize(group_size)
  trans.setInputMetaQuery(input_meta_query)
  res = trans.addTransformation()  # transformation is created here
  if not res['OK']:
    return res
  trans.setStatus("Active")
  trans.setAgentType("Automatic")
  trans_id = trans.getTransformationID()
  return trans_id


def submit_wms(job):
  """ Submit the job to the WMS
  @todo launch job locally
  """
  dirac = Dirac()
  input_data = ['/vo.cta.in2p3.fr/MC/PROD5b/Paranal/electron/sim_telarray/2432/Data/000xxx/electron_40deg_0deg_run104___cta-prod5b-paranal_desert-2147m-Paranal-dark.simtel.zst']
  job.setInputData(input_data)
  job.setJobGroup('EvnDispProd5')
  res = dirac.submitJob(job)
  if res['OK']:
    Script.gLogger.notice('Submitted job: ', res['Value'])
  return res


def launch_job(args):
  """ Simple launcher to instanciate a Job and setup parameters
      from positional arguments given on the command line.

      Parameters:
      args -- mode (trans_name dataset_name group_size)
  """
  DIRAC.gLogger.notice('Launching EventDisplay processing')
  # get arguments
  mode = args[0]

  if mode == 'TS':
    name_prefix = args[1]
    group_size = int(args[3])

    dataset_list = []
    if os.path.isfile(args[2]):
      DIRAC.gLogger.notice('Reading datasets from input file:', args[2])
      dataset_list = read_inputs_from_file(args[2])
    else:
      dataset_list.append(args[2])

  # use this mode for testing
  if mode == 'WMS':
    # create a job instance 
    job = EvnDispProd5SingJob(cpuTime=259200.)
    # set the version 
    job.version = 'eventdisplay-cta-dl1-prod5.v06'
    job.setName('Prod5b_EvnDisp')
    # set output SB
    job.setOutputSandbox(['*Log.txt'])
    job.base_path = '/vo.cta.in2p3.fr/user/a/arrabito'
    job.ts_task_id = '62'
    
    # just to test the metadata functionality
    # the following metadata values are arbitrary 
    simtel_meta_data = {'array_layout': 'Advanced-Baseline', 'site': 'LaPalma', 
                        'particle': 'proton', 'phiP': 0.0, 'thetaP': 20.0}
    job.set_meta_data(simtel_meta_data)
    job.set_file_meta_data(nsb=1)
    job.setupWorkflow(debug=True)
    res = submit_wms(job)

  # use this mode for production
  elif mode == 'TS':
    
    for dataset_name in dataset_list:
      job = EvnDispProd5SingJob(cpuTime=259200.)
      # set the version here
      job.version = 'eventdisplay-cta-dl1-prod5.v06'
      job.setName('Prod5b_EvnDisp')
      # set output SB
      job.setOutputSandbox(['*Log.txt'])

      # set the metadata
      input_meta_query = get_dataset_MQ(dataset_name)
      output_meta_data = copy(input_meta_query)
      job.set_meta_data(output_meta_data)
      job.set_file_meta_data(nsb=output_meta_data['nsb']['='])

      # define the job steps
      job.ts_task_id = '@{JOB_ID}'  # dynamic
      job.setupWorkflow(debug=False)
      job.setType('EvnDispProcessing')  # mandatory *here*

      # submit the transformations
      trans_name = name_prefix + '_' + dataset_name      
      res = submit_trans(job, trans_name, input_meta_query, group_size)
  else:
    DIRAC.gLogger.error('1st argument should be the submission mode: WMS or TS,\n\
                             not %s' % mode)
    return None

  return res


#########################################################
if __name__ == '__main__':

  arguments = Script.getPositionalArgs()
  if len(arguments) not in [1, 4]:
    Script.showHelp()
  try:
    res = launch_job(arguments)
    if not res['OK']:
      DIRAC.gLogger.error(res['Message'])
      DIRAC.exit(-1)
    else:
      DIRAC.gLogger.notice('Done')
  except Exception:
    DIRAC.gLogger.exception()
    DIRAC.exit(-1)



""" Class that contains client access to the Provenance DB handler. """

__RCSID__ = "$Id$"

# # from DIRAC
from DIRAC.Core.Base.Client import Client


class ProvClient(Client):

  """ Exposes the functionality available in the DataManagementSystem/ProvenanceManagerHandler
  """

  def __init__(self, url=None, **kwargs):
    """ Simple constructor
    """

    Client.__init__(self, **kwargs)
    res = self.serverURL = 'DataManagement/ProvenanceManager' if not url else url

  def addActivity(self, row):

    res = row.toJSON()
    rowJSON = res['Value']
    rpcClient = self._getRPC()
    return rpcClient.addActivity(rowJSON)

  def addDatasetEntity(self, row):

    res = row.toJSON()
    rowJSON = res['Value']
    rpcClient = self._getRPC()
    return rpcClient.addDatasetEntity(rowJSON)

  def addValueEntity(self, row):

    res = row.toJSON()
    rowJSON = res['Value']
    rpcClient = self._getRPC()
    return rpcClient.addValueEntity(rowJSON)

  def addUsed(self, row):

    res = row.toJSON()
    rowJSON = res['Value']
    rpcClient = self._getRPC()
    return rpcClient.addUsed(rowJSON)

  def addWasGeneratedBy(self, row):

    res = row.toJSON()
    rowJSON = res['Value']
    rpcClient = self._getRPC()
    return rpcClient.addWasGeneratedBy(rowJSON)

  def addAgent(self, row):

    res = row.toJSON()
    rowJSON = res['Value']
    rpcClient = self._getRPC()
    return rpcClient.addAgent(rowJSON)

  def addWasAttributedTo(self, row):

    res = row.toJSON()
    rowJSON = res['Value']
    rpcClient = self._getRPC()
    return rpcClient.addWasAttributedTo(rowJSON)

  def addWasAssociatedWith(self, row):

    res = row.toJSON()
    rowJSON = res['Value']
    rpcClient = self._getRPC()
    return rpcClient.addWasAssociatedWith(rowJSON)

  def addActivityDescription(self, row):

    res = row.toJSON()
    rowJSON = res['Value']
    rpcClient = self._getRPC()
    return rpcClient.addActivityDescription(rowJSON)

  def addDatasetDescription(self, row):

    res = row.toJSON()
    rowJSON = res['Value']
    rpcClient = self._getRPC()
    return rpcClient.addDatasetDescription(rowJSON)

  def addUsageDescription(self, row):

    res = row.toJSON()
    rowJSON = res['Value']
    rpcClient = self._getRPC()
    return rpcClient.addUsageDescription(rowJSON)

  def addGenerationDescription(self, row):

    res = row.toJSON()
    rowJSON = res['Value']
    rpcClient = self._getRPC()
    return rpcClient.addGenerationDescription(rowJSON)

  def addValueDescription(self, row):

    res = row.toJSON()
    rowJSON = res['Value']
    rpcClient = self._getRPC()
    return rpcClient.addValueDescription(rowJSON)

  def addWasConfiguredBy(self, row):

    res = row.toJSON()
    rowJSON = res['Value']
    rpcClient = self._getRPC()
    return rpcClient.addWasConfiguredBy(rowJSON)

  def addParameter(self, row):

    res = row.toJSON()
    rowJSON = res['Value']
    rpcClient = self._getRPC()
    return rpcClient.addParameter(rowJSON)

  def addConfigFile(self, row):

    res = row.toJSON()
    rowJSON = res['Value']
    rpcClient = self._getRPC()
    return rpcClient.addConfigFile(rowJSON)

  def addParameterDescription(self, row):

    res = row.toJSON()
    rowJSON = res['Value']
    rpcClient = self._getRPC()
    return rpcClient.addParameterDescription(rowJSON)

  def addConfigFileDescription(self, row):

    res = row.toJSON()
    rowJSON = res['Value']
    rpcClient = self._getRPC()
    return rpcClient.addConfigFileDescription(rowJSON)

  def getAgents(self):

    rpcClient = self._getRPC()
    return rpcClient.getAgents()

  def getAgentKey(self, agent_id):

    rpcClient = self._getRPC()
    return rpcClient.getAgentKey(agent_id)

  def getUsageDescription(self, activityDescription_id, role):

    rpcClient = self._getRPC()
    return rpcClient.getUsageDescription(activityDescription_id, role)

  def getParameterDescription(self, activityDescription_id, parameter_name):

    rpcClient = self._getRPC()
    return rpcClient.getParameterDescription(activityDescription_id, parameter_name)

  def getConfigFileDescription(self, activityDescription_id, configFile_name):

    rpcClient = self._getRPC()
    return rpcClient.getConfigFileDescription(activityDescription_id, configFile_name)

  def getActvityDescriptionKey(self, activityDescription_name, activityDescription_version):

    rpcClient = self._getRPC()
    return rpcClient.getActivityDescriptionKey(activityDescription_name, activityDescription_version)



""" Module to find, install and setup software packages
    J. Bregeon, L. Arrabito
    15/09/2019
"""

__RCSID__ = "$Id$"

# generic imports
import os
import glob
import shutil
import tarfile

# DIRAC imports
import DIRAC
from DIRAC.DataManagementSystem.Client.DataManager import DataManager
#from DIRAC.ConfigurationSystem.Client.Helpers.Operations import Operations
from CTADIRAC.Core.Utilities.tool_box import get_os_and_cpu_info


class SoftwareManager(object):
  """ Manage software setup
  """

  def __init__(self, soft_category):
    """ Constructor
    """
    self.CVMFS_DIR = '/cvmfs/sw.cta-observatory.org/software'
    self.LFN_ROOT = '/vo.cta.in2p3.fr/software'
    self.SOFT_CATEGORY_DICT = soft_category
    self.dm = DataManager()

  def _search_software(self, package, version, compiler):
    ''' Look for sotfware package
    '''
    # software package category
    category = self.SOFT_CATEGORY_DICT[package]
    # look for software on cvmfs
    package_dir = os.path.join(self.CVMFS_DIR, 'centos7',
                               compiler, category, package, version)
    if os.path.isdir(package_dir):
      DIRAC.gLogger.notice('Found package %s version %s at:\n%s' %
                           (package, version, package_dir))
      return DIRAC.S_OK({'Source': 'cvmfs', 'Path': package_dir})
    else:
      DIRAC.gLogger.notice('%s\n not found on cvmfs' % package_dir)
      # look for tarball in the Dirac file catalog
      package_dir = os.path.join(self.LFN_ROOT, 'centos7',
                                 compiler, category, package, version)
      DIRAC.gLogger.notice('Looking for tarball in %s' % package_dir)
      results = self.dm.getFilesFromDirectory(package_dir)
      try:
        first_file_path = results['Value'][0]
        if first_file_path[-7:] == '.tar.gz':
          results = self.dm.getActiveReplicas(first_file_path)
          if results['OK']:
            return DIRAC.S_OK({'Source': 'tarball', 'Path': package_dir})
      except BaseException:
        DIRAC.gLogger.warn('No usual tarball found in the directory')

    return DIRAC.S_ERROR('Could not find package %s / %s / %s in any location'
                         % (package, version, compiler))

  def find_software(self, package, version, compiler='gcc48_default'):
    """ check if the software package is installed in any software area
      Keyword arguments:
      package -- package name as the directory name
      version -- software version as the directory name
      compiler -- compiler version and configuration
    """
    # first check if cvmfs is available
    # Not used anymore -> First try cvmfs and then try tarball
    #ops_helper = Operations()
    #use_cvmfs = ops_helper.getValue('SoftwarePolicy/UseCvmfs', bool)
    #DIRAC.gLogger.notice('SoftwarePolicy for UseCvmfs is:', use_cvmfs)

    # get platform and cpu information
    try:
      platform, cpu_name, inst = get_os_and_cpu_info()
      DIRAC.gLogger.notice('Running %s on %s (%s)' % (platform, cpu_name, inst))
    except BaseException:
      inst = 'noOpt'
      DIRAC.gLogger.notice('Could not determine platform and cpu information')

    req_inst = compiler.split('_')[1]
    if req_inst == 'default':
      results = self._search_software(package, version, compiler)
      return results
    elif req_inst == 'noOpt':
      results = self._search_software(package, version, compiler)
      return results
    elif req_inst == 'sse4':
      if inst in ['sse4', 'avx', 'avx2']:
        results = self._search_software(package, version, compiler)
        return results
      else:
        DIRAC.gLogger.warn('CPU has no sse4 instructions, running non optimized version')
        match_compiler = compiler.replace(req_inst, 'noOpt')
        results = self._search_software(package, version, match_compiler)
        return results
    elif req_inst == 'avx':
      if inst in ['avx', 'avx2']:
        results = self._search_software(package, version, compiler)
        return results
      else:
        DIRAC.gLogger.warn('CPU has no avx instructions, running non optimized version')
        match_compiler = compiler.replace(req_inst, 'noOpt')
        results = self._search_software(package, version, match_compiler)
        return results
    elif req_inst == 'avx2':
      if inst in ['avx2']:
        results = self._search_software(package, version, compiler)
        return results
      else:
        DIRAC.gLogger.warn('CPU has no avx2 instructions, running non optimized version')
        match_compiler = compiler.replace(req_inst, 'noOpt')
        results = self._search_software(package, version, match_compiler)
        return results
    elif compiler == 'gcc83_avx512':
      if inst in ['avx512']:
        results = self._search_software(package, version, compiler)
        return results
      else:
        DIRAC.gLogger.warn('CPU has no avx512 instructions, running non optimized version')
        match_compiler = compiler.replace(req_inst, 'noOpt')
        results = self._search_software(package, version, match_compiler)
        return results
    elif req_inst == 'matchcpu':
      match_compiler = compiler.replace(req_inst, inst)
      if match_compiler == 'gcc48_avx512':
        DIRAC.gLogger.warn('%s not available for gcc48' % inst)
        DIRAC.gLogger.warn('Using gcc83 avx512 instead')
        match_compiler = 'gcc83_avx512'
      results = self._search_software(package, version, match_compiler)
      return results
    else:
      DIRAC.S_ERROR('Unknown compiler specified: %s' % compiler)
    return DIRAC.S_ERROR('Could not find package %s version %s / %s in any location'
                         % (package, version, compiler))

  def install_dirac_scripts(self, package_dir):
    """ copy DIRAC scripts in the current directory
    """
    dirac_scripts = glob.glob(os.path.join(package_dir, 'dirac_*'))
    try:
      for one_file in dirac_scripts:
        shutil.copy2(one_file, '.')
      return DIRAC.S_OK()
    except shutil.Error as error:
      return DIRAC.S_ERROR('Failed to install DIRAC scripts:\n%s' % error)

  def dump_setup_script_path(self, package_dir, textfilename='setup_script_path.txt'):
    """ dump the path to setupPackage.sh in a one line ascii file
          to be read and source by the following script
    """
    script_path = os.path.join(package_dir, 'setupPackage.sh')
    open(textfilename, 'w').writelines(script_path + '\n')
    return DIRAC.S_OK()

  def install_software(self, tar_lfn, target_dir='.'):
    """ install software package in the current directory
    """
    DIRAC.gLogger.notice('Installing package at %s' % tar_lfn)

    # Download the tar file
    DIRAC.gLogger.notice('Trying to download package:', tar_lfn)
    res = self.dm.getFile(tar_lfn)
    if not res['OK']:
      return res

    if tar_lfn in res['Value']['Successful']:
      DIRAC.gLogger.notice(' Package downloaded successfully:', tar_lfn)
    else:
      error = 'Failed to download package:', tar_lfn
      return DIRAC.S_ERROR(error)

    # Extract the tar file to the target directory
    tar_mode = "r|*"
    tar = tarfile.open(os.path.basename(tar_lfn), tar_mode)
    for tarInfo in tar:
      tar.extract(tarInfo, target_dir)
    tar.close()
    os.unlink(os.path.basename(tar_lfn))
    # Done
    DIRAC.gLogger.notice('Package %s installed successfully at:\n%s'
                         % (tar_lfn, target_dir))

    return DIRAC.S_OK(target_dir)



#!/usr/bin/env python
"""
  Simple terminal job monitoring
"""
__RCSID__ = "$Id$"

from DIRAC import gLogger
from DIRAC.Core.Utilities.DIRACScript import DIRACScript as Script
from CTADIRAC.Core.Utilities import tool_box
from CTADIRAC.Core.Utilities.tool_box import highlight

@Script()
def main():
  ''' Select jobs based on conditions
  '''
  Script.registerSwitch("", "owner=", "the job owner")
  Script.registerSwitch("", "jobGroup=", "the job group")
  Script.registerSwitch("", "hours=", "Get status for jobs of the last n hours")
  Script.registerSwitch("", "failed=", "1 or 0 : Save or not failed jobs in \"failed.txt\"")
  switches, argss = Script.parseCommandLine(ignoreErrors=True)
  # defaults
  owner = "arrabito"
  job_group = ""
  n_hours = 24

  for switch in switches:
    if switch[0].lower() == "owner":
      owner = switch[1]
    elif switch[0].lower() == "jobgroup":
      job_group = switch[1]
    elif switch[0].lower() == "hours":
      n_hours = int(switch[1])

  # do the jobs via the 2 main methods
  jobs_list = tool_box.get_job_list(owner, job_group, n_hours)
  gLogger.notice(
      "%s jobs found for group \"%s\" and owner \"%s\" in the past %s hours\n" %
      (len(jobs_list), job_group, owner, n_hours))

  # get status dictionary
  status_dict, sites_dict = tool_box.parse_jobs_list(jobs_list)

  # print out my favourite tables
  gLogger.notice(
      "%16s\tWaiting\tRunning\tFailed\tStalled\tDone\tTotal" %
      "Site")
  for key, val in sites_dict.items():
    txt = "%16s\t%s\t%s\t%s\t%s\t%s\t%s" %\
        (key.split('LCG.')[-1], val['Waiting'], val['Running'], val['Failed'],
         val['Stalled'], val['Done'], val['Total'])
    if float(val['Done']) > 0.:
      # More than 10% crash, print bold red
      if float(val['Failed']) / float(val['Done']) > 0.1:
        txt = highlight(txt)
    gLogger.notice(txt)

  gLogger.notice("%16s\t%s\t%s\t%s\t%s\t%s\t%s" %
                        ('Total', status_dict['Waiting'], status_dict['Running'],
                         status_dict['Failed'], status_dict['Stalled'],
                         status_dict['Done'], status_dict['Total']))

if __name__ == '__main__':
  main()


""" SimpleCtapipe script to create a Transformation

@authors: J. Bregeon, L. Arrabito, D. Landriu, J. Lefaucheur
            April 2018
"""

__RCSID__ = "$Id$"

from DIRAC.Core.Base import Script

Script.setUsageMessage('\n'.join([__doc__.split('\n')[1],
                                  'Usage:',
                                  '  %s [option|cfgfile] ...' % Script.scriptName]))

# switches for job configuration
Script.registerSwitch('', 'backend=', 'Backend: TS,WMS,paramWMS default=WMS')
Script.registerSwitch('', 'trans_name=', 'Transformation name, mandatory for backend=TS')
Script.registerSwitch('', 'group_size=', 'Nb of input files per job, default=1')
#  switches for ctapipe
Script.registerSwitch('', 'config_file=', 'Configuration file, mandatory')
Script.registerSwitch('', 'output_type=', 'Data level output, e.g. DL1 or DL2, str, mandatory')
Script.registerSwitch('', 'max_events=', 'Maximal number of events to be processed, int, optional')

Script.parseCommandLine()

import DIRAC
from DIRAC.TransformationSystem.Client.Transformation import Transformation
from DIRAC.TransformationSystem.Client.TransformationClient import TransformationClient
from CTADIRAC.Interfaces.API.SimpleCtapipeJob import SimpleCtapipeJob
from DIRAC.Interfaces.API.Dirac import Dirac
from DIRAC.Core.Workflow.Parameter import Parameter
from DIRAC.Resources.Catalog.FileCatalogClient import FileCatalogClient
from CTADIRAC.Core.Utilities.tool_box import read_inputs_from_file

# Default values
backend = 'WMS'
trans_name = None
group_size = 1
config_file = None
max_events = 10000000000
output_type = None

for switch in Script.getUnprocessedSwitches():
  if switch[0].lower() == "backend":
    backend = switch[1]
  if switch[0].lower() == "trans_name":
    trans_name = switch[1]
  elif switch[0].lower() == "group_size":
    group_size = int(switch[1])
  elif switch[0].lower() == "config_file":
    config_file = switch[1]
  elif switch[0].lower() == "output_type":
    output_type = switch[1]
  elif switch[0].lower() == "max_events":
    max_events = switch[1]

# Mandatory switches
if output_type is None:
  DIRAC.gLogger.error('Missing --output_type argument')
  DIRAC.exit(-1)
elif output_type not in ['DL1', 'DL2']:
  DIRAC.gLogger.error('Wrong --output_type')
  Script.showHelp()
  DIRAC.exit(-1)

if config_file is None:
  DIRAC.gLogger.error('Missing --config_file argument')
  DIRAC.exit(-1)

if backend == 'TS' and trans_name is None:
  DIRAC.gLogger.error('Missing --trans_name argument')
  DIRAC.exit(-1)


def submit_trans(job, trans_name, infileList, group_size):
  """ Create a transformation executing the job workflow
  """
  DIRAC.gLogger.notice('submit_trans : %s' % trans_name)

  # Initialize JOB_ID
  job.workflow.addParameter(Parameter("JOB_ID", "000000", "string", "", "",
                                      True, False, "Temporary fix"))

  trans = Transformation()
  trans.setTransformationName(trans_name)  # this must be unique
  trans.setType("DataReprocessing")
  trans.setDescription("SimpleCtapipe example")
  trans.setLongDescription("ctapipe classify, reconstruct and merge: calib_imgreco")  # mandatory
  trans.setBody(job.workflow.toXML())
  trans.setGroupSize(group_size)

  res = trans.addTransformation()  # transformation is created here
  if not res['OK']:
    DIRAC.gLogger.error('Error creating transformation: ', res['Message'])
    return res

  transID = res['Value']

  # Activate the transformation
  trans.setStatus("Active")
  trans.setAgentType("Automatic")

  # Add files to the transformation
  tc = TransformationClient()
  res = tc.addFilesToTransformation(transID, infileList)
  if not res['OK']:
    DIRAC.gLogger.error('Error adding files to the transformation: ', res['Message'])

  return res


def submit_paramWMS(job, infileList):
  """ Submit parametric jobs to the WMS  """

  job.setParameterSequence('InputData', infileList, addToWorkflow='ParametricInputData')
  job.setOutputData(['*.h5'], outputPath='ctapipe_data2')
  job.setName('ctapipejob')

  dirac = Dirac()
  res = dirac.submitJob(job)

  if res['OK']:
    Script.gLogger.notice('Submission Result: ', res['Value'])

  return res


def submit_WMS(job, infileList):
  """ Submit the job locally or to the WMS
  """
  dirac = Dirac()
  job.setInputData(infileList)
  res = dirac.submitJob(job)
  if res['OK']:
    Script.gLogger.notice('Submission Result: ', res['Value'])
  return res


def run_ctapipe():
  """ Simple wrapper to create a SimpleCtapipeJob and setup parameters
      from switches given on the command line
  """
  # init FileCatalogClient
  fc = FileCatalogClient()

  # create and common job configuration
  job = SimpleCtapipeJob(cpuTime=432000)
  job.setName('ctapipe job')
  job.version = 'v0.6.2'
  job.setType('DataReprocessing')  # Needed for job meshing. Check Operations/CTA/JobTypeMapping on the CS
  job.setOutputSandbox(['*Log.txt'])

  # configure ctapipe job from config_file and command line switches
  job.setConfig(config_file)
  job.output_type = output_type
  job.max_events = max_events

  # select the input file according to config_file parameters
  input_file = job.getInputfile()
  # read list of input file names
  input_file_list = read_inputs_from_file(input_file)
  # get the metadata of the inputs
  res = fc.getFileUserMetadata(input_file_list[0])
  if not res['OK']:
    DIRAC.gLogger.error("Failed to get user metadata:", res['Message'])
    return res
  meta_data_dict = res['Value']
  # set meta data for ctapipe outputs
  job.setCtapipeMD(meta_data_dict)
  # add the sequence of the xecutables
  job.setupWorkflow(debug=True)

  # submit parametric jobs to WMS
  if backend == 'paramWMS':
    res = submit_paramWMS(job, input_file_list)
  # submit a single job to WMS for debug
  if backend == 'WMS':
    res = submit_WMS(job, input_file_list)
  # submit to the Transformation System
  elif backend == 'TS':
    res = submit_trans(job, trans_name, input_file_list, group_size)

  return res


#########################################################
if __name__ == '__main__':

  try:
    res = run_ctapipe()
    if not res['OK']:
      DIRAC.gLogger.error(res['Message'])
      DIRAC.exit(-1)
    else:
      DIRAC.gLogger.notice('Done')
  except Exception:
    DIRAC.gLogger.exception()
    DIRAC.exit(-1)



"""
  Simple Wrapper on the Job class to run a ctapipe modeling job
"""

__RCSID__ = "$Id$"

# generic imports
import json

# DIRAC imports
from CTADIRAC.Interfaces.API.Prod5Stage1Job import Prod5Stage1Job


class Prod5CtaPipeModelingJob(Prod5Stage1Job):
    """ Job extension class for ctapipe stage1 modeling processing,
    """

    def __init__(self, cpuTime=432000):
        """ Constructor

    Keyword arguments:
    cpuTime -- max cpu time allowed for the job
    """
        Prod5Stage1Job.__init__(self)
        self.setCPUTime(cpuTime)
        # defaults
        self.version = 'v0.15.0'
        self.setName('ctapipe_modeling')
        self.program_category = 'stage1'
        self.prog_name = 'ctapipe-modeling'
        self.ctapipe_base_dl2_config = 'base_dl2_config.yml'
        self.ctapipe_site_config = 'prod5b_paranal_alpha_nectarcam.yml'
        self.configuration_id = 7
        self.output_data_level = 2

    def setupWorkflow(self, debug=False):
        """ Setup job workflow by defining the sequence of all executables
        All parameters shall have been defined before that method is called.
    """
        i_step = 0
        # step 1 -- debug
        if debug:
            ls_step = self.setExecutable('/bin/ls -alhtr', logFile='LS_Init_Log.txt')
            ls_step['Value']['name'] = 'Step%i_LS_Init' % i_step
            ls_step['Value']['descr_short'] = 'list files in working directory'
            i_step += 1

        # step 2
        sw_step = self.setExecutable('cta-prod-setup-software',
                                     arguments='-p %s -v %s -a stage1 -g %s' %
                                               (self.package, self.version, self.compiler),
                                     logFile='SetupSoftware_Log.txt')
        sw_step['Value']['name'] = 'Step%i_SetupSoftware' % i_step
        sw_step['Value']['descr_short'] = 'Setup software'
        i_step += 1

        # step 3 run stage1
        ev_step = self.setExecutable('./dirac_run_ctapipe_modeling',
                                     arguments=f"--base_config {self.ctapipe_base_dl2_config} \
                                             --site_config {self.ctapipe_site_config} \
                                             --input_ext {self.simtel_ext}",
                                     logFile='ctapipe_stage1_Log.txt')
        ev_step['Value']['name'] = 'Step%i_ctapipe_stage1' % i_step
        ev_step['Value']['descr_short'] = 'Run ctapipe stage 1'
        i_step += 1

        # step 4 set meta data and register Data
        meta_data_json = json.dumps(self.metadata)
        file_meta_data_json = json.dumps(self.file_meta_data)

        meta_data_field = {'array_layout': 'VARCHAR(128)', 'site': 'VARCHAR(128)',
                           'particle': 'VARCHAR(128)',
                           'phiP': 'float', 'thetaP': 'float',
                           self.program_category + '_prog': 'VARCHAR(128)',
                           self.program_category + '_prog_version': 'VARCHAR(128)',
                           'data_level': 'int', 'configuration_id': 'int', 'merged': 'int'}
        meta_data_field_json = json.dumps(meta_data_field)

        # register Data
        data_output_pattern = './Data/*.h5'  # %self.output_data_level
        dm_step = self.setExecutable('cta-prod-managedata',
                                     arguments="'%s' '%s' '%s' %s '%s' %s %s '%s' Data" %
                                               (meta_data_json, meta_data_field_json,
                                                file_meta_data_json,
                                                self.base_path, data_output_pattern, self.package,
                                                self.program_category, self.catalogs),
                                     logFile='DataManagement_Log.txt')
        dm_step['Value']['name'] = 'Step%s_DataManagement' % i_step
        dm_step['Value']['descr_short'] = 'Save data files to SE and register them in DFC'
        i_step += 1

        # step 5 register Log
        log_file_pattern = './Data/*.log_and_prov.tgz'
        file_meta_data = {}
        file_meta_data_json = json.dumps(file_meta_data)
        log_step = self.setExecutable('cta-prod-managedata',
                                      arguments="'%s' '%s' '%s' %s '%s' %s %s '%s' Log" %
                                                (meta_data_json, meta_data_field_json,
                                                 file_meta_data_json,
                                                 self.base_path, log_file_pattern, self.package,
                                                 self.program_category, self.catalogs),
                                      logFile='LogManagement_Log.txt')
        log_step['Value']['name'] = 'Step%s_LogManagement' % i_step
        log_step['Value']['descr_short'] = 'Save log to SE and register them in DFC'
        i_step += 1

        # step 6 failover step
        failover_step = self.setExecutable('/bin/ls -l',
                                           modulesList=['Script', 'FailoverRequest'])
        failover_step['Value']['name'] = 'Step%s_Failover' % i_step
        failover_step['Value']['descr_short'] = 'Tag files as unused if job failed'
        i_step += 1

        # Step 7 - debug only
        if debug:
            ls_step = self.setExecutable('/bin/ls -Ralhtr', logFile='LS_End_Log.txt')
            ls_step['Value']['name'] = 'Step%s_LSHOME_End' % i_step
            ls_step['Value']['descr_short'] = 'list files in Home directory'
            i_step += 1



#!/bin/env python
"""
  Simple terminal job error summary
"""

import DIRAC
from DIRAC import gLogger
from DIRAC.Core.Utilities.DIRACScript import DIRACScript as Script
from CTADIRAC.Core.Utilities import tool_box

@Script()
def main():
  ''' Select jobs based on conditions
  '''
  Script.registerSwitch("", "owner=", "the job owner")
  Script.registerSwitch("", "jobGroup=", "the job group")
  Script.registerSwitch("", "hours=", "Get status for jobs of the last n hours")
  Script.registerSwitch("", "failed=", "1 or 0 : Save or not failed jobs in \"failed.txt\"")
  switches, argss = Script.parseCommandLine(ignoreErrors=True)

  # defaults
  owner = "arrabito"
  job_group = ""
  n_hours = 24
  save_failed = False

  for switch in switches:
    if switch[0].lower() == "owner":
      owner = switch[1]
    elif switch[0].lower() == "jobgroup":
      job_group = switch[1]
    elif switch[0].lower() == "hours":
      n_hours = int(switch[1])
    elif switch[0].lower() == "failed":
      save_failed = int(switch[1])

  # Start doing something
  # import Dirac here after parseCommandLine
  from DIRAC.Interfaces.API.Dirac import Dirac
  dirac = Dirac()

  # do the jobs via the 2 main methods
  jobs_list = tool_box.get_job_list(owner, job_group, n_hours)
  gLogger.notice(
      "%s jobs found for group \"%s\" and owner \"%s\" in the past %s hours\n" %
      (len(jobs_list), job_group, owner, n_hours))

  # get jobs status
  status = dirac.getJobStatus(jobs_list)

  # print out my favourite tables
  SitesDict = {}

  for job in jobs_list:
    #    print job, status['Value'][int(job)]
    site = status['Value'][int(job)]['Site']
  #    site=status['Value'][int(job)]['CE']
    minstatus = status['Value'][int(job)]['MinorStatus']
    majstatus = status['Value'][int(job)]['Status']

    if majstatus not in {'Done', 'Failed'}:
      continue

    if site not in SitesDict.keys():
      if site.find('.') == -1:
        site = '    None'  # note that blank spaces are needed
      SitesDict[site] = {'Total': 0, 'Failed': 0, 'Errors': {}}

    SitesDict[site]['Total'] += 1
    if majstatus == 'Failed':
      SitesDict[site]['Failed'] += 1
      if minstatus not in SitesDict[site]['Errors'].keys():
        SitesDict[site]['Errors'][minstatus] = 0
      SitesDict[site]['Errors'][minstatus] += 1

  gLogger.notice("%20s  Finish  Errors  Rate  Failure reason" % "Site")
  for site, val in sorted(SitesDict.items()):
    errstr = ""
    for error, amount in val['Errors'].items():
      if len(errstr) > 0:
        errstr += "\n\t\t\t\t\t    "
      errstr += "%s (%d)" % (error, amount)

    txt = "%20s%8d%8d%5d%%  %s" % (site, val['Total'], val['Failed'], val['Failed'] * 100 / val['Total'], errstr)
    gLogger.notice(txt)

  # saved information on failed jobs on disk
  if save_failed:
    txt = ''
    for job in jobs_list:
      majstatus = status['Value'][int(job)]['Status']
      if majstatus == "Failed":
        txt += str(dirac.getJobSummary(int(job))) + '\n'
    open('failed.txt', 'w').write(txt)
    gLogger.notice('Saved failed.txt on disk')

  DIRAC.exit()

if __name__ == '__main__':
  main()


#!/usr/bin/env python
'''
Remove a given dataset

Usage:
   cta-prod-remove-dataset <datasetName or ascii file with a list of datasets>
'''

__RCSID__ = "$Id$"

import os
import DIRAC
from DIRAC import gLogger
from DIRAC.Core.Utilities.DIRACScript import DIRACScript as Script
from CTADIRAC.Core.Utilities.tool_box import read_inputs_from_file
from DIRAC.Resources.Catalog.FileCatalogClient import FileCatalogClient

@Script()
def main():
    Script.parseCommandLine(ignoreErrors=True)
    argss = Script.getPositionalArgs()
    fc = FileCatalogClient()
    datasetList = []
    if len(argss) == 1:
      if os.path.isfile(argss[0]):
        gLogger.notice('Reading datasets from input file: %s' % argss[0])
        datasetList = read_inputs_from_file(argss[0])
      else:
        datasetList.append(argss[0])
    else:
      Script.showHelp()

    for dataset in datasetList:
      result = fc.removeDataset(dataset)

      if not result['OK']:
        gLogger.error("Failed to remove %s: %s" % (dataset, result['Message']))
        DIRAC.exit(-1)
      else:
        gLogger.notice("Successfully removed dataset", dataset)

    DIRAC.exit()

####################################################
if __name__ == '__main__':
  main()


#############################################################################
# $HeadURL$
#############################################################################

""" The StorageMonitorAgent checks the storage space usage
    for SEs defined in the Agent section
"""

__RCSID__ = "$Id$"

from DIRAC import S_OK, S_ERROR
from DIRAC.Core.Base.AgentModule import AgentModule
from DIRAC.FrameworkSystem.Client.NotificationClient import NotificationClient
from DIRAC.ConfigurationSystem.Client.CSAPI import CSAPI
from DIRAC.Core.Utilities.Subprocess import systemCall
from DIRAC.Core.Utilities.PrettyPrint import printTable


class StorageMonitorAgent(AgentModule):

  addressTo = ''
  addressFrom = ''
  voName = ''
  bdii = 'lcg-bdii.cern.ch:2170'
  productionSEs = []
  subject = "StorageMonitorAgent"

  def initialize(self):

    self.addressTo = self.am_getOption('MailTo', self.addressTo)
    self.addressFrom = self.am_getOption('MailFrom', self.addressFrom)

    if self.addressTo and self.addressFrom:
      self.log.info("MailTo", self.addressTo)
      self.log.info("MailFrom", self.addressFrom)

    self.productionSEs = self.am_getOption('ProductionSEs', self.productionSEs)
    if self.productionSEs:
      self.log.info("ProductionSEs", self.productionSEs)
    else:
      self.log.fatal("ProductionSEs option not defined for agent")
      return S_ERROR()

    self.voName = self.am_getOption('VirtualOrganization', self.voName)
    if self.voName:
      self.log.info("Agent will manage VO", self.voName)
    else:
      self.log.fatal("VirtualOrganization option not defined for agent")
      return S_ERROR()

    self.bdii = self.am_getOption('Bdii', self.bdii)
    if self.bdii:
      self.log.info("Bdii", self.bdii)

    self.csAPI = CSAPI()
    return self.csAPI.initialize()

  def execute(self):
    """ General agent execution method
    """
    # Get a "fresh" copy of the CS data
    result = self.csAPI.downloadCSData()
    if not result['OK']:
      self.log.warn("Could not download a fresh copy of the CS data", result['Message'])

    # Execute command to retrieve storage usage information
    cmdTuple = ['lcg-infosites', '--vo', self.voName, 'se']
    self.log.info("Executing %s" % cmdTuple)
    ret = systemCall(0, cmdTuple, env={'LCG_GFAL_INFOSYS': self.bdii})

    if not ret['OK']:
      return ret
    elif not ret['Value'][1] != '':
      self.log.error("Error while executing %s" % cmdTuple)
      return S_ERROR()

    # initialize sedict
    sedict = {}
    for SE in self.productionSEs:
      sedict[SE] = []

    fields = ['SE', 'Available(TB)', 'Used(TB)', 'Total(TB)', 'Used(%)']
    records = []
    fullSEList = []

    for se in ret['Value'][1].split('\n'):
      if len(se.split()) == 4:
        spacedict = {}
        SE = se.split()[3]
        if SE in self.productionSEs and se.split()[0] != 'n.a' and se.split()[1] != 'n.a':
          # ## convert into TB
          available = float(se.split()[0]) / 1e9
          used = float(se.split()[1]) / 1e9
          spacedict['Available'] = available
          spacedict['Used'] = used
          spacedict['Total'] = available + used
          sedict[SE].append(spacedict)

    for SE in self.productionSEs:
      for spacedict in sedict[SE]:
        available = '%.1f' % spacedict['Available']
        used = '%.1f' % spacedict['Used']
        total = '%.1f' % spacedict['Total']
        fraction_used = spacedict['Used'] / spacedict['Total'] * 100
        if fraction_used > 90.:
          fullSEList.append(SE)
          self.log.warn("%s full at %.1f%%" % (SE, fraction_used))
        fraction_used = '%.1f' % fraction_used
        records.append([SE, available, used, total, fraction_used])

    body = printTable(fields, records, printOut=False)

    if len(fullSEList) > 0:
      self.subject = 'CRITICAL storage usage beyond 90%%: %s' % (', '.join(fullSEList))

    if self.addressTo and self.addressFrom:
      notification = NotificationClient()
      result = notification.sendMail(self.addressTo, self.subject, body, self.addressFrom, localAttempt=False)
      if not result['OK']:
        self.log.error('Can not send  notification mail', result['Message'])

    return S_OK()



#!/usr/bin/env python
"""
  Add files to an existing transformation

  Usage:
    cta-transformation-add-files <transID> <ascii file with lfn list>
"""

__RCSID__ = "$Id$"

import DIRAC
from DIRAC.Core.Utilities.DIRACScript import DIRACScript as Script
from DIRAC.TransformationSystem.Client.TransformationClient import TransformationClient
from CTADIRAC.Core.Utilities.tool_box import read_inputs_from_file

@Script()
def main():
    Script.parseCommandLine()
    args = Script.getPositionalArgs()
    if (len(args) != 2):
      Script.showHelp()

    # get arguments
    transID = args[0]
    infile = args[1]
    infileList = read_inputs_from_file(infile)

    tc = TransformationClient()
    res = tc.addFilesToTransformation(transID, infileList)  # Files added here

    if not res['OK']:
      DIRAC.gLogger.error(res['Message'])
      DIRAC.exit(-1)
    else:
      DIRAC.gLogger.notice("Successfully added %d files to transformation %s" % (len(infileList), transID))
      DIRAC.exit(0)

if __name__ == "__main__":
  main()



#!/usr/bin/env python
"""
Dump in a file the list of files for a given dataset

Usage:
   cta-prod-dump-dataset <datasetName>
"""

__RCSID__ = "$Id$"

import DIRAC
from DIRAC import gLogger
from DIRAC.Core.Utilities.DIRACScript import DIRACScript as Script
from DIRAC.Resources.Catalog.FileCatalogClient import FileCatalogClient

@Script()
def main():
  Script.parseCommandLine(ignoreErrors=True)
  argss = Script.getPositionalArgs()
  fc = FileCatalogClient()

  if len(argss) > 0:
    datasetName = argss[0]
  else:
    Script.showHelp()

  result = fc.getDatasetFiles(datasetName)

  if not result['OK']:
    gLogger.error("Failed to get files for dataset:", result['Message'])
    DIRAC.exit(-1)
  else:
    lfnList = result['Value']['Successful'][datasetName]

    f = open(datasetName + '.list', 'w')
    for lfn in lfnList:
      f.write(lfn + '\n')
    f.close()
    gLogger.notice('%d files have been put in %s.list' % (len(lfnList), datasetName))
    DIRAC.exit()

####################################################
if __name__ == '__main__':
  main()